/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Starting process instances        */
/*                                  asynchronously                    */
/*                                                                    */
/* This sample program shows how to start process instances           */
/* asynchronously and receive responses sent by the server.           */
/**********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification


int main (int argc, char ** argv)
{

  FmcjExecutionService                  service;
  FmcjCorrelID                          correlID;

  unsigned long                         i               = 0;
  int                                   j               = 0;

  string                                choice          ="";
  int                                   usrchoice       = 0;

/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/
  if (argc!=3)
  {
    cout << "\nUsage: " << argv[0] << " <UserID> <Password>" << endl;
    return -1;
  }
  cout << endl;
  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();
  rc = service.Logon( argv[1], argv[2], FmcjService::Default );
  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - " << result->MessageText()
         << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return rc;
  }
  cout << "Logged on with UserID '" << service.UserID()
       << "' on system '" << service.SystemName()
       << "' at system group '" << service.SystemGroupName()
       << "'." << endl;

/*-------------------------------------------------------------------*/
/* Start PEA to permit automatically running processes to execute    */
/* activity implementations                                          */
/*-------------------------------------------------------------------*/
  cout << "\nStarting PEA for user '" << service.UserID() << "'... " << endl;
  rc = service.PEAStartUp();
  if (rc!=FMC_OK && rc!=FMC_ERROR_ALREADY_STARTED )
  {
    cout << "Starting PEA for user '" << service.UserID() << "' failed with RC "
         << rc << " - " << result->MessageText() << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return rc;
  }
  // Shutdown PEA later only if started by this program
  bool PEAstarted=false;
  if (rc==FMC_ERROR_ALREADY_STARTED)
  {
    cout << "PEA is already started." << endl;
    PEAstarted=false;
  }
  else PEAstarted=true;


/*-------------------------------------------------------------------*/
/* Query for available process templates                             */
/*-------------------------------------------------------------------*/
  vector<FmcjProcessTemplate>           tList;
  string                                PTListSort      ="NAME ASC";

  rc = service.QueryProcessTemplates( FmcjNoFilter,
                                      &PTListSort,
                                      FmcjNoThreshold,
                                      tList );

  if (rc != FMC_OK)
  {
    cout << "Querying process templates failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return rc;
  }

/*-------------------------------------------------------------------*/
/* Create and start process instances asynchronously                 */
/*-------------------------------------------------------------------*/
  do                                    // Beginning of main loop
  {
    cout << "\nAvailable process templates:\n" << endl;
    cout << "#   Template name\n-----------------" << endl;

    if (tList.size() > 0 )
    {
      for( i=0; i<tList.size(); i++ )
      {
        cout << i;
        if (i<10) cout << " ";
        cout << "  ";
        cout << tList[i].Name() << endl;
      }
    }
    else cout << "    <none>" << endl;

    do
    {
      cout << "\nWhich process do you want to be executed asynchronously? ";
      cout.flush();
      cin >> choice;
      j=0;
      for (i=0; i<strlen(choice.c_str()); i++)
      { if (isdigit(choice.c_str()[i])) j++; }
    } while (i!=j);

    i = atoi(choice.c_str());
    cout << endl;

    if (i>(tList.size()-1) && tList.size()>0)
    {
      cout << "Please use range 0-" << tList.size()-1 << "." << endl;
    }
    else
    {
      string instanceName    ="Example_NOP_Process";
      string usercontext     ="individualism";

      // Initialize correlationID before each asynchonous request
      memcpy( correlID, FMCJ_NO_CORRELID, 24 );

      /*-------------------------------------------------------------------*/
      /* Start an instance of the process asynchronously, giving it a name */
      /* No input container is provided in this sample                     */
      /* If the instance requires input data the values are not set        */
      /* If an instance with this name exists, the server returns an error */
      /* The usercontext can be used to correlate the server response      */
      /*-------------------------------------------------------------------*/
      rc = tList[i].ExecuteProcessInstanceAsync( &instanceName,
                                                 0,
                                                 0,
                                                 true,
                                                 &correlID,
                                                 &usercontext );

      // Analyze return code of ExecuteProcessInstanceAsync command

      if (rc!=FMC_OK)
      {
        cout << "\nExecuteProcessInstanceAsync() call failed!\n" << "RC: " << rc << " - "
             << result->MessageText() << endl;
      }
      else
      {
        cout << "\nExecuteProcessInstanceAsync() requested successfully." << endl
             << "CorrelID: " << correlID << endl;
      }


      // Main receive loop. Everytime an ExecutionData object is pushed
      // from the server, information about it is displayed onscreen.

      do
      {

        /*-------------------------------------------------------------------*/
        /* Listen to the server for any data from the server.                */
        /* Timeout after timeoutRec milliseconds - needs to be adjusted.     */
        /*-------------------------------------------------------------------*/
        signed long          timeoutRec = 20000;
        FmcjExecutionData    recData;

        cout << "\nListening to server... \n"; cout.flush();

        // Receive any data that is pushed to this session
        memcpy( correlID, FMCJ_NO_CORRELID, 24 );

        rc = service.Receive( &correlID, recData, timeoutRec);

        if (rc == FMC_OK)
        {
          // The request has been processed by the server
          // Examine the received ExecutionData object
          cout << "Received ExecutionData object\n"
               << "                       CorrelID    : "
               << correlID << endl
               << "                       Type        : ";
          switch (recData.Kind())
          {
            case FmcjExecutionData::NotSet: cout << "NotSet" << endl; break;
            case FmcjExecutionData::Error: cout << "Error" << endl; break;
            case FmcjExecutionData::Terminate: cout << "Terminate" << endl; break;
            case FmcjExecutionData::ItemDeleted: cout << "ItemDeleted" << endl; break;
            case FmcjExecutionData::ItemUpdated: cout << "ItemUpdated" << endl; break;
            case FmcjExecutionData::Workitem: cout << "Workitem" << endl; break;
            case FmcjExecutionData::ActivityInstanceNotification: cout << "ActivityInstanceNotification" << endl; break;
            case FmcjExecutionData::ProcessInstanceNotification: cout << "ProcessInstanceNotification" << endl; break;
            case FmcjExecutionData::ExecuteInstanceResponse: cout << "ExecuteInstanceResponse" << endl; break;
            case FmcjExecutionData::ExecuteProgramResponse: cout << "ExecuteProgramResponse" << endl; break;
          }
          cout << "                       User context: ";
          if (recData.UserContextIsNull()) cout << "<not set>" << endl;
          else cout << recData.UserContext() << endl;

          // If the server responds an error
          if (recData.IsError())
          {
            FmcjError       * recError;

            cout << "\nError information\n"
                 << "-----------------\n";

            recError = recData.ErrorFromData();

            cout << "RC  : " << recError->Rc() << "\n"
                 << "Text: " << recError->MessageText() << endl;
          }

          // If the server response is as expected
          if ( (recData.Kind()==FmcjExecutionData::ExecuteInstanceResponse)
            && (recData.IsError()==false) )
          {
            FmcjReadOnlyContainer                 * recContainer;

            // Access the process instance's output container

            cout << "                       Output Container name: ";

            recContainer = recData.ReadOnlyContainerFromData();

            cout << recContainer->Type() << endl;
          }


        } /* end if: FMC_OK */

        if (rc==FMC_ERROR_TIMEOUT)
        {
          // No response from the server.
          // The process instance may have been started but not yet terminated

          cout << "Timeout occured.\n" << endl;
          cout << "Now checking status of asynchronously started example process instance..." << endl;

          vector<FmcjProcessInstance> iList;
          string                      PListFilter ="NAME='Example_NOP_Process'";

          rc = service.QueryProcessInstances( &PListFilter,
                                              FmcjNoSortCriteria,
                                              FmcjNoThreshold,
                                              iList );
          if (rc != FMC_OK)
          {
            cout << "Querying process instances failed with RC "
                 << rc << " - " << result->MessageText() << endl;
          }

          if (iList.size()==0)
          {
            /*-------------------------------------------------------------*/
            /* Possible reasons:                                           */
            /* 1. The server has not got the request due to a              */
            /*    communication problem.                                   */
            /* 2. The instance has terminated and is already deleted.      */
            /*    Retry to receive the response, check the audit trail     */
            /*-------------------------------------------------------------*/
            cout << "The process instance object does not exist (anymore)." << endl;
          }
          else
          {
            /*-------------------------------------------------------------*/
            /* The process instance was created by the server.             */
            /* Check its state.                                            */
            /*-------------------------------------------------------------*/
            FmcjReadOnlyContainer  prcContainer;
            vector<FmcjWorkitem>   wList;
            string                 WListFilter;

            switch (iList[0].State())
            {
              case Fmc_PS_Finished:

                cout << "The process instance was finished successfully." << endl;

                // Access the output container
                rc = iList[0].OutContainer(prcContainer);
                cout << "\nOutput Container name: ";
                cout << prcContainer.Type() << endl;
                break;


              case Fmc_PS_Deleted:

                cout << "The process instance was deleted." << endl;

                // Access the output container
                rc = iList[0].OutContainer(prcContainer);
                cout << "\nOutput Container name: ";
                cout << prcContainer.Type() << endl;
                break;


              case Fmc_PS_Running:

                /*--------------------------------------------------------*/
                /* The process instance is still running.                 */
                /* Maybe the timeout value was too short for this process.*/
                /* Maybe an activity is in error and holds the process up.*/
                /*--------------------------------------------------------*/
                cout << "The process instance is still running." << endl;

                WListFilter =
                 "PROCESS_NAME='Example_NOP_Process' AND STATE IN (IN_ERROR)";

                // Check the workitems of the process instance for errors
                rc = service.QueryWorkitems( &WListFilter,
                                             FmcjNoSortCriteria,
                                             FmcjNoThreshold,
                                             wList );
                if (rc != FMC_OK)
                {
                  cout << "Querying workitems failed with RC "
                       << rc << " - " << result->MessageText() << endl;
                }

                cout << "The following workitems are in an erroneous state:\n" << endl;

                if (wList.size()>0)
                {
                  FmcjError     error;

                  for (i=0; i<wList.size(); i++)
                  {
                    error = wList[i].ErrorReason();
                    cout << wList[i].Name();
                    cout << "    Error RC  : " << error.Rc() << endl;
                    cout << "    Error Text: " << error.MessageText() << endl;
                  }
                }
                else cout << "   <no workitems in error>" << endl;
                break;


              default:

                cout << "The process instance is in state " << iList[0].State() << "." << endl;
                break;
            }
          }
        }

        cout << "\nReceive again [y/n]? ";
        cout.flush();

        cin >> choice;

        if (strncmp(choice.c_str(),"n",1)==0) usrchoice=255;

      } while(usrchoice!=255);


    }

  } while (usrchoice!=255);


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  if (PEAstarted)
  {
    cout << "\nShutting down PEA." << endl;
    rc = service.PEAShutDown();
  }
  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  delete(result);
  return 0;
}
