/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Tutorial - Monitoring processes              */
/*                                                                    */
/* This sample program shows how to monitor a process with its        */
/* subordinated programs, blocks and subprocesses                     */
/**********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification

int main (int argc, char ** argv)
{

  int                                   i                = 0;
  int                                   j                = 0;

  int                                   aiIndex         = 0;
  int                                   cciIndex        = 0;

  FmcjExecutionService                  service;

  FmcjProcessInstance                   processInstance;
  vector<FmcjProcessInstance>           viList;

  FmcjActivityInstance                  activityInstance;
  vector<FmcjActivityInstance>          aList;

  FmcjControlConnectorInstance          ccInstance;
  vector<FmcjControlConnectorInstance>  ccList;

  FmcjInstanceMonitor                   piMonlevel0;
  FmcjInstanceMonitor                 * piMonlevel1;
  FmcjInstanceMonitor                 * biMon;

  FmcjPoint                             point;
  vector<FmcjPoint>                     pointList;

  string                                PIListFilter    ="";
  string                                PIListSort      ="NAME ASC";


  string                                choice          ="";
  int                                   usrchoice       = 0;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    cout << "\nUsage: " << argv[0] << " <UserID> <Password>" << endl;
    return -1;
  }
  cout << endl;
  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();
  rc = service.Logon( argv[1], argv[2], FmcjService::Default );
  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - " << result->MessageText()
         << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }
  cout << "Logged on with UserID '" << service.UserID()
       << "' on system '" << service.SystemName()
       << "' at system group '" << service.SystemGroupName()
       << "'." << endl;


/*-------------------------------------------------------------------*/
/* Query available process instances                                 */
/*-------------------------------------------------------------------*/

  do                                    // Beginning of main loop
  {
    // Clear vector

    viList.erase(viList.begin(),viList.end());

    rc = service.QueryProcessInstances( FmcjNoFilter,
                                        &PIListSort,
                                        FmcjNoThreshold,
                                        viList );
    if (rc != FMC_OK)
    {
      cout << "Querying process instances failed with RC " << rc << " - "
           << result->MessageText() << endl;
      service.Logoff();
      // Deinit API environment
      FmcjGlobal::Disconnect();
      delete(result);
      return -1;
    }
    cout << "#   State        Instance name" << endl;
    cout << "------------------------------" << endl;
    if (viList.size()>0)
    {
      for( i=0; i<viList.size(); i++ )
      {
        cout << i;
        if (i<10) cout << " ";
        cout << "  ";
        switch (viList[i].State())
        {
          case FmcjProcessInstance::undefined: cout << "undefined  "; break;
          case FmcjProcessInstance::ready: cout << "ready      "; break;
          case FmcjProcessInstance::running: cout << "running    "; break;
          case FmcjProcessInstance::finished: cout << "finished   "; break;
          case FmcjProcessInstance::terminated: cout << "terminated "; break;
          case FmcjProcessInstance::suspended: cout << "suspended  "; break;
          case FmcjProcessInstance::terminating: cout << "terminating"; break;
          case FmcjProcessInstance::suspending: cout << "suspending "; break;
          case FmcjProcessInstance::deleted: cout << "deleted    "; break;
        }
        cout << "  " << viList[i].Name() << endl;
      }
    }
    else cout << "    <none>" << endl;

    do
    {
      cout << "\nPlease enter the instance number you want to view the monitor of: ";
      cout.flush();
      cin >> choice;
      j=0;
      for (i=0; i<strlen(choice.c_str()); i++)
      { if (isdigit(choice.c_str()[i])) j++; }
    } while (i!=j);

    i = atoi(choice.c_str());
    cout << endl;

    if (i>(viList.size()-1) && viList.size()>0)
    {
      cout << "Please use range 0-" << viList.size()-1 << "." << endl;
    }
    else
    {
      // Obtain monitor of selected process instance

      processInstance = viList[i];

      rc = processInstance.ObtainProcessMonitor( piMonlevel0, false );
      if (rc != FMC_OK)
      {
        cout << "Obtaining monitor failed with RC " << rc << " - "
             << result->MessageText() << endl;
        service.Logoff();
        // Deinit API environment
        FmcjGlobal::Disconnect();
        delete(result);
        return -1;
      }

      // Show all activity instances and control connector instances
      // (clear vectors first)

      aList.erase(aList.begin(),aList.end());
      ccList.erase(ccList.begin(),ccList.end());

      piMonlevel0.ActivityInstances(aList);

      piMonlevel0.ControlConnectorInstances( ccList );

      cout << "\n#   Type       Activity instance name\n"
           << "-------------------------------------\n";

      // Show activity instances
      if (aList.size() > 0 )
      {
        for( aiIndex=0; aiIndex<aList.size(); aiIndex++ )
        {
          activityInstance = aList[aiIndex];

          cout << aiIndex;
          if (aiIndex<10) cout << " ";
          cout << "  ";

          switch (activityInstance.Kind())
          {
           case FmcjActivityInstance::NotSet: cout << "undefined"; break;
           case FmcjActivityInstance::Process: cout << "process  "; break;
           case FmcjActivityInstance::Program: cout << "program  "; break;
           case FmcjActivityInstance::Block: cout << "block    "; break;
          }
          cout << "  " << activityInstance.Name() << endl;
        }
      }
      else cout << "    <none>" << endl;

      // Show control connector instances

      cout << "\n#   State          Control connector instance name\n"
           << "--------------------------------------------------\n";

      if (ccList.size() > 0 )
      {
        for( cciIndex=aiIndex; cciIndex<ccList.size()+aiIndex; cciIndex++ )
        {
          ccInstance = ccList[cciIndex-aiIndex];

          cout << cciIndex;
          if (cciIndex<10) cout << " ";
          cout << "  ";

          switch (ccInstance.State())
          {
           case FmcjControlConnectorInstance::False: cout << "false        "; break;
           case FmcjControlConnectorInstance::True: cout << "true         "; break;
           case FmcjControlConnectorInstance::NotEvaluated: cout << "not evaluated"; break;
           case FmcjControlConnectorInstance::NotSet: cout << "not set      "; break;
          }
          if (!ccInstance.NameIsNull())
               cout << "  " << ccInstance.Name() << endl;
          else cout << "  <not set>" << endl;
        }
        cciIndex--;
      }
      else cout << "    <none>" << endl;

      do
      {
        cout << "\nWhich activity/control connector instance do you want to examine closer? ";
        cout.flush();
        cin >> choice;
        j=0;
        for (i=0; i<strlen(choice.c_str()); i++)
        { if (isdigit(choice.c_str()[i])) j++; }
      } while (i!=j);

      i = atoi(choice.c_str());
      cout << endl;

      if ( i>cciIndex || i<0 )
      {
        cout << "Please use range 0-" << cciIndex << "." << endl;
      }
      else
      {
        if (i<aiIndex)
        {
          // Examine activity instance closer

          activityInstance = aList[i];

          switch (activityInstance.Kind())
          {
            case FmcjActivityInstance::NotSet:
              cout << "\nBecause of an unknown reason, this activity instance has not been initialised."
                   << endl;
              break;

            case FmcjActivityInstance::Program:

              // Showing some program information

              cout << "\nActivity instance - program information\n"
                   << "---------------------------------------\n";
              cout << "State          : ";
              switch (activityInstance.State())
              {
                case FmcjActivityInstance::undefined: cout << "undefined    "; break;
                case FmcjActivityInstance::ready: cout << "ready        "; break;
                case FmcjActivityInstance::finished: cout << "finished     "; break;
                case FmcjActivityInstance::suspended: cout << "suspended    "; break;
                case FmcjActivityInstance::checkedOut: cout << "checkedOut   "; break;
                case FmcjActivityInstance::executed: cout << "executed     "; break;
                case FmcjActivityInstance::forceFinished: cout << "forceFinished"; break;
                case FmcjActivityInstance::terminating: cout << "terminating  "; break;
                case FmcjActivityInstance::running: cout << "running      "; break;
                case FmcjActivityInstance::terminated: cout << "terminated   "; break;
                case FmcjActivityInstance::inError: cout << "inError      "; break;
                case FmcjActivityInstance::planning: cout << "planning     "; break;
                case FmcjActivityInstance::deleted: cout << "deleted      "; break;
                case FmcjActivityInstance::suspending: cout << "suspending   "; break;
                case FmcjActivityInstance::inactive: cout << "inactive   "; break;
                case FmcjActivityInstance::skipped: cout << "skipped   "; break;
               }
               cout << "\nImplementation : "
                    << activityInstance.Implementation() << endl
                    << "Activation time: "
                    << string(activityInstance.ActivationTime()) << endl;
              break;

            case FmcjActivityInstance::Process:

              // Fetch the corresponding process instance monitor

              cout << "\nSubprocess instance monitor information\n"
                   << "---------------------------------------\n";

              piMonlevel1 = piMonlevel0.ObtainProcessMonitor ( activityInstance );
              rc = result->Rc();
              if (rc == FMC_ERROR_DOES_NOT_EXIST)
              {
                cout << "   A process instance of this subprocess does not exist yet,\n"
                     << "   hence it can't be monitored!" << endl;
              }
              else if (rc != FMC_OK)
              {
                cout << "Obtaining monitor failed with RC " << rc << " - "
                     << result->MessageText() << endl;
                service.Logoff();
                // Deinit API environment
                FmcjGlobal::Disconnect();
                delete(result);
                return -1;
              }
              else
              {

                // Clear vectors

                aList.erase(aList.begin(),aList.end());
                ccList.erase(ccList.begin(),ccList.end());

                piMonlevel1->ActivityInstances(aList);
                piMonlevel1->ControlConnectorInstances( ccList );

                cout << "Number of subordinate activity instances         : "
                     << aList.size() << "\n"
                     << "Number of subordinate control connector instances: "
                     << ccList.size() << "\n" << endl;
              }

              break;

            case FmcjActivityInstance::Block:

              // Fetch the corresponding block instance monitor

              cout << "\nBlock instance monitor information\n"
                   << "----------------------------------\n";

              biMon = piMonlevel0.ObtainBlockMonitor( activityInstance );
              rc = result->Rc();
              if (rc != FMC_OK)
              {
                cout << "Obtaining monitor failed with RC " << rc << " - "
                     << result->MessageText() << endl;
                service.Logoff();
                // Deinit API environment
                FmcjGlobal::Disconnect();
                delete(result);
                return -1;
              }

              // Clear vectors

              aList.erase(aList.begin(),aList.end());
              ccList.erase(ccList.begin(),ccList.end());

              biMon->ActivityInstances(aList);
              biMon->ControlConnectorInstances(ccList);

              cout << "Number of subordinate activity instances         : "
                   << aList.size() << "\n"
                   << "Number of subordinate control connector instances: "
                   << ccList.size() << "\n" << endl;
              break;
          }

        }


        else
        {
          // Examine control connector instance closer

          ccInstance = ccList[i-aiIndex];

          cout << "\nControl connector instance information\n"
               << "--------------------------------------\n";
          cout << "Source activity Oid : "
               << ccInstance.PersistentOidOfSourceActivity() << "\n"
               << "Target activity Oid : "
               << ccInstance.PersistentOidOfTargetActivity() << "\n"
               << "Transition Condition: "
               << ccInstance.TransitionCondition() << "\n"
               << "Bend points:\n";

          // Clear points vector first
          pointList.erase(pointList.begin(),pointList.end());

          ccInstance.BendPoints( pointList );

          if (pointList.size()>0)
          {
            for (j=0; j<pointList.size(); j++)
            {
              point = pointList[j];
              cout << "   [" << j << "] " << "X: " << point.XPosition()
                   << " Y: " << point.YPosition() << endl;
            }
          }
          else
          {
            cout << "   <no bend points exist>" << endl;
          }

        }

      }

    } /* end else */

    cout << endl << "Do you want to restart or quit [r/q]? ";
    cout.flush();

    cin >> choice;

    if (strncmp(choice.c_str(),"q",1)==0) usrchoice=255;

  } while (usrchoice!=255);


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  delete(result);
  return 0;
}
