/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2003
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Handling process templates        */
/*                                                                    */
/* This sample program shows how to start process instances from an   */
/* existing process template.                                         */
/**********************************************************************/

#include <fmcjcrun.h>                   /* MQWF C API interface       */
#include <ctype.h>                      /* character classification   */
#include <stdio.h>                      /* for standard in-output     */
#include <stdlib.h>                     /*                            */
#include <string.h>                     /* string conversion          */
#define INFOSIZE 4097



int main (int argc, char ** argv)
{

  FmcjExecutionServiceHandle        service     = 0;
  FmcjProcessTemplateVectorHandle   tList       = 0;
  FmcjProcessTemplateHandle         templ       = 0;
  FmcjResultHandle                  result      = 0;
  FmcjReadWriteContainerHandle      inContainer = 0;
  FmcjProcessInstanceHandle         processInstance = 0;

  unsigned long                     i           = 0;
  unsigned long                     j           = 0;
  unsigned long                     threshold   = 10;
  unsigned long                     vectSize    = 0;

  APIRET                            rc          = FMC_OK;

  char                              choice[1]         = "";
  char                              tInfo[INFOSIZE]   = "";

  int                               usrchoice   = 0;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    printf("\nUsage: %s %s %s\n\n",argv[0],"<UserID>","<Password>");
    return -1;
  }
  fflush(stdout);
  rc = FmcjGlobalConnect();
  result = FmcjResultObjectOfCurrentThread();

  rc = FmcjExecutionServiceAllocate( &service );
  if (rc != FMC_OK)
  {
    printf("Service object could not be allocated: %u\n",rc);
    FmcjGlobalDisconnect();
    return -1;
  }

  rc = FmcjExecutionServiceLogon(service, argv[1], argv[2],
                                 Fmc_SM_Default,Fmc_SA_NotSet );
  if (rc != FMC_OK)
  {
    printf("Logon failed with RC  %u %s %s\n",rc," - ",
                      FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);

    /* Deinit API environment */
    rc = FmcjExecutionServiceDeallocate( &service );
    FmcjGlobalDisconnect();
    return -1;
  }
  printf("Logged on with UserID '%s' ",
             FmcjExecutionServiceUserID(service, tInfo, INFOSIZE));
  printf("on system '%s' ",
             FmcjExecutionServiceSystemName(service, tInfo, INFOSIZE));
  printf("at system group '%s'.\n",
             FmcjExecutionServiceSystemGroupName(service, tInfo, INFOSIZE));
  fflush(stdout);


/*--------------------------------------------------------------------*/
/* Display current template list, query user for action               */
/*--------------------------------------------------------------------*/

  do /* Beginning of main tool loop*/
  {
     /* Display as many process templates as specified with threshold */

    printf("\nAvailable process templates:\n\n");

    rc = FmcjExecutionServiceQueryProcessTemplates(service,
                                                   FmcjNoFilter,
                                                   "NAME ASC",
                                                   &threshold,
                                                   &tList );

    if (rc != FMC_OK)
    {
      printf("Querying process templates failed with RC %u-%s\n",
             rc,FmcjResultMessageText(result, tInfo, INFOSIZE));
      fflush(stdout);

      FmcjProcessTemplateVectorDeallocate(&tList);
      FmcjExecutionServiceLogoff(service);
      /* Deinit API environment */
      rc= FmcjExecutionServiceDeallocate( &service );
      FmcjGlobalDisconnect();
      return -1;
    }

    printf("    Input container\n");
    printf(" #  needed?           Template name\n");
    printf("-----------------------------------\n");
    fflush(stdout);

    vectSize = FmcjProcessTemplateVectorSize(tList);
    if (vectSize > 0 )
    {
      for( i=0; i< vectSize; i++ )
      {
        if (i < 10) printf(" %u",i);
        else        printf("%u",i);

        templ = FmcjProcessTemplateVectorNextElement(tList);

        if (FmcjProcessTemplateInContainerNeeded(templ))
             printf("  yes               ");
        else printf("  no                ");
        printf ("%s\n",FmcjProcessTemplateName(templ,tInfo,INFOSIZE));
        fflush(stdout);
        rc = FmcjProcessTemplateDeallocate(&templ);
      }
    }
    else printf("  <no templates available>\n");

    fflush(stdout);
    printf("\nCommands: [v] View template info                    [c] Create an instance\n");
    printf("          [s] Create and start an instance          [q] Quit program\n");
    printf("[v/s/c/q]? ");
    fflush(stdout);

    fgets(choice,3,stdin);

    usrchoice=0;

    if (strncmp(choice,"v",1)==0) usrchoice=1;
    if (strncmp(choice,"s",1)==0) usrchoice=2;
    if (strncmp(choice,"c",1)==0) usrchoice=3;
    if (strncmp(choice,"q",1)==0) usrchoice=255;

    if (vectSize>0)

    switch (usrchoice)
    {

     case 1:
/*-------------------------------------------------------------------*/
/* Shows some attributes of the selected process template.           */
/* If secondary attributes are accessed the API refreshes the        */
/* template object automatically from the server.                    */
/*-------------------------------------------------------------------*/
       do
       {
         if ((vectSize-1) < 100)
           printf("Please enter template number (0-%u%s",vectSize-1,") ");
         else
           printf("Please enter template number (1-99)");
         fflush(stdout);

         scanf("%2[^\n]",tInfo);
         j=0;
         for (i=0; i < strlen(tInfo); i++)
         { if (isdigit(tInfo[i])) j++; }
         if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       } while (i!=j);
       i = atoi(tInfo);
       fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
       printf("\n");
       fflush(stdout);
       if ( i > (vectSize-1))
       {
         printf("Please use range 0-%u%s\n",vectSize-1,".");
       }
       else
       {
         /* Get the selected template */
         templ = FmcjProcessTemplateVectorFirstElement(tList);
         for (j = 0; j < i; j++)
         {
           rc = FmcjProcessTemplateDeallocate(&templ);
           templ = FmcjProcessTemplateVectorNextElement(tList);
         }

         printf("Template properties after Refresh():\n");
         printf("-------------------------------------\n");
         printf("   (P) Name     : %s\n",
                     FmcjProcessTemplateName(templ,tInfo,INFOSIZE));
         printf("   (P) Category : %s\n",
                     FmcjProcessTemplateCategory(templ,tInfo,INFOSIZE));
         printf("   (P) Object id: %s\n",
                     FmcjProcessTemplatePersistentOid(templ,tInfo,INFOSIZE));
         printf("   (S) Process administrator: %s\n",
                     FmcjProcessTemplateProcessAdmin(templ,tInfo,INFOSIZE));
         printf("   (S) Input container name : %s\n",
                     FmcjProcessTemplateInContainerName(templ,tInfo,INFOSIZE));
         printf("   (S) Output container name: %s\n",
                     FmcjProcessTemplateOutContainerName(templ,tInfo,INFOSIZE));

         rc = FmcjProcessTemplateDeallocate(&templ);
       } /* end else: user's choice is in range */

       break;


     case 2:
/*-------------------------------------------------------------------*/
/* Demonstrates how to create and start a process instance.          */
/*-------------------------------------------------------------------*/
        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter template number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter template number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if ( i > (vectSize-1))
        {
           printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else                            /* user's choice is in range */
        {
          /* Get the selected template */
          templ = FmcjProcessTemplateVectorFirstElement(tList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjProcessTemplateDeallocate(&templ);
            templ = FmcjProcessTemplateVectorNextElement(tList);
          }

          if (FmcjProcessTemplateInContainerNeeded(templ))
          {
                /* Query for input container values only for "CreditRequest" */

            if (!strcmp(FmcjProcessTemplateName(templ,tInfo,INFOSIZE),
                                                "CreditRequest"))
            {
              rc = FmcjProcessTemplateInitialInContainer(templ, &inContainer);
              if (rc) {
               inContainer = 0;
               rc = 0;
              }
              else
              {
                printf("FirstName: ");
                fflush(stdout);
                scanf("%50[^\n]",tInfo);

                rc = FmcjReadWriteContainerSetStringValue(inContainer,
                                                       "FirstName",tInfo);
                fgets(tInfo,INFOSIZE,stdin); /* clear the stdin */

                printf("LastName : ");
                fflush(stdout);
                scanf("%50[^\n]",tInfo);

                rc = FmcjReadWriteContainerSetStringValue(inContainer,
                                                       "LastName",tInfo);
                fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */

              }

            }
          }

          /* Create and start instance, passing an input container */
          rc = FmcjProcessTemplateCreateAndStartInstance(templ,
                                            0, 0, 0,
                                            inContainer,
                                            false,
                                            &processInstance);
          /* Check if instance was created and started successfully */
          if (rc != FMC_OK)
          {
            printf("Creating new instance failed with RC  %u %s %s\n",
                   rc," - ", FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);

            rc = FmcjProcessTemplateDeallocate(&templ);
            FmcjProcessTemplateVectorDeallocate(&tList);
            FmcjExecutionServiceLogoff(service);
            /* Deinit API environment */
            rc = FmcjExecutionServiceDeallocate( &service );
            FmcjGlobalDisconnect();
            return -1;
          }

        rc = FmcjProcessInstanceDeallocate(&processInstance);
        rc = FmcjProcessTemplateDeallocate(&templ);
        } /* end else: user's choice is in range */

        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Demonstrates how to create a process instance                     */
/*-------------------------------------------------------------------*/
        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter template number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter template number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
          {
            printf("Please use range 0-%u%s\n",vectSize-1,".");
          }
        else
        {
          /* Get the selected template */
          templ = FmcjProcessTemplateVectorFirstElement(tList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjProcessTemplateDeallocate(&templ);
            templ = FmcjProcessTemplateVectorNextElement(tList);
          }

          /* Create an instance without starting it */

          rc = FmcjProcessTemplateCreateInstance(templ,
                                                  0, 0, 0,
                                                  false,
                                                  &processInstance);
          /* Check if instance was created successfully */
          if (rc != FMC_OK)
          {
            printf("Creating new instance failed with RC  %u %s %s\n",
                     rc," - ", FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);

            rc = FmcjProcessTemplateDeallocate(&templ);
            FmcjProcessTemplateVectorDeallocate(&tList);
            FmcjExecutionServiceLogoff(service);
            /* Deinit API environment */
            rc = FmcjExecutionServiceDeallocate( &service );
            FmcjGlobalDisconnect();

            return -1;
          }

          rc = FmcjProcessInstanceDeallocate(&processInstance);
          rc = FmcjProcessTemplateDeallocate(&templ);
        } /* end else: user's choice is in range */

        break;

      }                                 /* end of switch */
      else printf("No templates available!\n");

      fflush(stdout);
      FmcjProcessTemplateVectorDeallocate(&tList);

  } while ( (usrchoice!=255) );         /* end of main tool loop */


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  printf("\nLogging off.\n");
  rc = FmcjExecutionServiceLogoff(service);

  rc= FmcjExecutionServiceDeallocate( &service );

  printf("Deinitializing API environment.\n");
  rc = FmcjGlobalDisconnect();

  return 0;
}
