/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQS Workflow API Examples - Handling workitems and notifications   */
/*                                                                    */
/* This program shows how to handle workitems and notifications.      */
/**********************************************************************/

#include <fmcjcrun.h>                   /* MQWF C API interface       */
#include <ctype.h>                      /* character classification   */
#include <stdio.h>                      /* for standard in-output     */
#include <stdlib.h>                     /*                            */
#include <string.h>                     /* string conversion          */
#define INFOSIZE 4097


int main (int argc, char ** argv)
{
  FmcjExecutionServiceHandle                    service   = 0;
  FmcjResultHandle                              result    = 0;
  FmcjWorkitemVectorHandle                      vwList    = 0;
  FmcjProcessInstanceNotificationHandle         piNot     = 0;
  FmcjProcessInstanceNotificationVectorHandle   piNotList = 0;
  FmcjActivityInstanceNotificationHandle        aiNot     = 0;
  FmcjActivityInstanceNotificationVectorHandle  aiNotList = 0;
  FmcjStringVectorHandle                        info      = 0;
  FmcjWorkitemHandle                            workitem  = 0;
  FmcjReadOnlyContainerHandle                   hdlInput  = 0;
  FmcjReadWriteContainerHandle                  hdlOutput = 0;
  FmcjErrorHandle                               error     = 0;
  FmcjCDateTime                                 timeValue;

  char           aInfo          [INFOSIZE]                = "";
  char           choice         [1]                       = "";
  char           queriedObjects [4]                       = "";
  char           tInfo          [INFOSIZE]                = "";


  int            usrchoice               = 0;

  unsigned long  vectSize                = 0;
  unsigned long  infoVectSize            = 0;

  APIRET         rc                      = 0;
  unsigned long          i               = 0;
  unsigned long          j               = 0;
  bool           PEAstarted              = false;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    printf("\nUsage: %s %s %s\n\n",argv[0],"<UserID>","<Password>");
    return -1;
  }
  fflush(stdout);
  rc = FmcjGlobalConnect();
  result = FmcjResultObjectOfCurrentThread();

  rc = FmcjExecutionServiceAllocate( &service );
  if (rc != FMC_OK)
  {
    printf("Service object could not be allocated: %u\n",rc);
    FmcjGlobalDisconnect();
    return -1;
  }
  rc = FmcjExecutionServiceLogon(service, argv[1], argv[2],
                                 Fmc_SM_Default,Fmc_SA_NotSet );

  if (rc != FMC_OK)
  {
    printf("Logon failed with RC  %u %s %s\n",rc," - ",
                      FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);

    /* Deinit API environment */
    rc = FmcjExecutionServiceDeallocate( &service );
    FmcjGlobalDisconnect();
    return rc;
  }
  printf("\nLogged on with UserID '%s' ",
             FmcjExecutionServiceUserID(service, tInfo, INFOSIZE));
  printf("on system '%s' ",
             FmcjExecutionServiceSystemName(service, tInfo, INFOSIZE));
  printf("at system group '%s'.\n",
             FmcjExecutionServiceSystemGroupName(service, tInfo, INFOSIZE));
  fflush(stdout);

/*-------------------------------------------------------------------*/
/* Start a PEA to permit starting workitems                          */
/*-------------------------------------------------------------------*/

  printf("\nStarting PEA for user '%s'...\n",
             FmcjServiceUserID(service, tInfo, INFOSIZE));

  rc = FmcjExecutionServicePEAStartUp(service);
  if (rc!=FMC_OK && rc!=FMC_ERROR_ALREADY_STARTED )
  {
    printf("Starting PEA for user '%s' failed with RC %u-%s\n",
           FmcjServiceUserID(service, tInfo, INFOSIZE), rc,
           FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);

    /* Deinit API environment */
    FmcjExecutionServiceLogoff(service);
    rc = FmcjExecutionServiceDeallocate( &service );
    FmcjGlobalDisconnect();
    return -1;
  }
  if (rc==FMC_ERROR_ALREADY_STARTED)
  {
    printf("PEA is already started.\n");
    fflush(stdout);
    PEAstarted=false;
  }
  else PEAstarted=true;


/*-------------------------------------------------------------------*/
/* Based on user's choice, display available workitems, process      */
/* instance notifications or activity instance notifications.        */
/*-------------------------------------------------------------------*/

  do                                    /* Beginning of main loop */
  {
    printf("\nDo you want to view the available [w]orkitems?\n");
    printf("                                  [p]rocess instance notifications?\n");
    printf("                                  [a]ctivity instance notifications?\n");
    printf("[w/p/a]? ");
    fflush(stdout);

    fgets(choice,3,stdin);

    if (strncmp(choice,"w",1)==0)
    {
      strcpy(queriedObjects, "wi");

      rc = FmcjExecutionServiceQueryWorkitems(service,
                       "OWNER=CURRENT_USER",
                                   "NAME ASC",
                                   FmcjNoThreshold,
                                   &vwList);
      if (rc != FMC_OK)
      {
        printf("Querying workitems failed with RC %u-%s\n",
               rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
        fflush(stdout);

        rc = FmcjWorkitemVectorDeallocate(&vwList);
        FmcjExecutionServiceLogoff(service);
        /* Deinit API environment */
        rc= FmcjExecutionServiceDeallocate( &service );
        FmcjGlobalDisconnect();
        return -1;
      }

      printf("\n                 Support\n");
      printf(" #  State        Tool?    Workitem name\n");
      printf("-----------------------------------------\n");
      fflush(stdout);

      vectSize = FmcjWorkitemVectorSize(vwList);

      if (vectSize > 0)
      {
        for( i=0; i < vectSize; i++ )
        {
          if (i < 10) printf(" %u  ",i);
          else        printf("%u  ",i);

          workitem = FmcjWorkitemVectorNextElement(vwList);

          switch (FmcjWorkitemState(workitem))
          {
            case Fmc_IS_NotSet:       printf("undefined    "); break;
            case Fmc_IS_Ready:        printf("ready        "); break;
            case Fmc_IS_Finished:     printf("finished     "); break;
            case Fmc_IS_Suspended:    printf("suspended    "); break;
            case Fmc_IS_CheckedOut:   printf("checkedOut   "); break;
            case Fmc_IS_Executed:     printf("executed     "); break;
            case Fmc_IS_ForceFinished:printf("forceFinished"); break;
            case Fmc_IS_Terminating:  printf("terminating  "); break;
            case Fmc_IS_Running:      printf("running      "); break;
            case Fmc_IS_Terminated:   printf("terminated   "); break;
            case Fmc_IS_Disabled:     printf("disabled     "); break;
            case Fmc_IS_InError:      printf("inError      "); break;
            case Fmc_IS_Planning:     printf("planning     "); break;
            case Fmc_IS_Deleted:      printf("deleted      "); break;
            case Fmc_IS_Suspending:   printf("suspending   "); break;
            case Fmc_IS_Expired:      printf("expired      "); break;
            default:                  printf("%d",FmcjWorkitemState(workitem));
          }
          if (FmcjWorkitemSupportToolsIsNull(workitem)) printf("no ");
          else                                                      printf("yes");

          printf("      %s\n",FmcjWorkitemName(workitem,tInfo,INFOSIZE));
          fflush(stdout);
          rc = FmcjWorkitemDeallocate(&workitem);
        }
      }
      else printf("   <no workitems for user '%s' found>\n",
                FmcjServiceUserID(service, tInfo, INFOSIZE));

      /* Show actions which user can perform */
      printf("\nCommands: [v] View detailed workitem info       [i] Finish workitem\n");
      printf("          [s] Start workitem                    [c] Force finish workitem\n");
      printf("          [r] Restart workitem                  [o] Start support tool\n");
      printf("          [f] Force restart workitem            [n] Reload a list\n" );
      printf("          [t] Terminate workitem                [q] Quit program\n");
      printf("          [d] Delete workitem                                   \n");
      printf("[v/s/r/f/t/i/c/o/n/q/d]? ");
      fflush(stdout);

    } /* end if: display workitems */

    if (strncmp(choice,"p",1)==0)
    {
      strcpy(queriedObjects, "pin");
      rc = FmcjExecutionServiceQueryProcessInstanceNotifications(service,
                                       FmcjNoFilter,
                                                      "NAME ASC",
                                                      FmcjNoThreshold,
                                      &piNotList);
      if (rc != FMC_OK)
      {
        printf("Querying process instance notifications failed with RC %u-%s\n",
               rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
        fflush(stdout);

        rc = FmcjProcessInstanceNotificationVectorDeallocate(&piNotList);
        FmcjExecutionServiceLogoff(service);
        /* Deinit API environment */
        rc= FmcjExecutionServiceDeallocate( &service );
        FmcjGlobalDisconnect();
        return -1;
      }
      printf("\n#   Creation Time     Notification for\n");
      printf("--------------------------------------\n");
      fflush(stdout);

      vectSize = FmcjProcessInstanceNotificationVectorSize(piNotList);

      if (vectSize > 0 )
      {
        for( i=0; i < vectSize; i++ )
        {
          if (i < 10) printf(" %u  ",i);
          else        printf("%u  ",i);

          piNot = FmcjProcessInstanceNotificationVectorNextElement(piNotList);
          timeValue = FmcjProcessInstanceNotificationCreationTime(piNot);

          printf("%s  %s\n", FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE),
                FmcjProcessInstanceNotificationName(piNot, tInfo, INFOSIZE));

          rc = FmcjProcessInstanceNotificationDeallocate(&piNot);
        }
      }
      else printf("   <no process instance notifications for user '%s' found>\n",
                FmcjServiceUserID(service, tInfo, INFOSIZE));

      /* Show actions which user can perform */
      printf("\nCommands: [p] View process instance              [n] Reload a list\n");
      printf("              notification info                  [q] Quit program\n");
      printf("[p/n/q]? ");
      fflush(stdout);

    } /* end if: display process instance notifications */

    if (strncmp(choice,"a",1)==0)
    {
      strcpy(queriedObjects, "ain");
      rc = FmcjExecutionServiceQueryActivityInstanceNotifications(service,
                                           FmcjNoFilter,
                                                       "NAME ASC",
                                                       FmcjNoThreshold,
                                                       &aiNotList );
      if (rc != FMC_OK)
      {
        printf("Querying activity instance notifications failed with RC %u-%s\n",
               rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
        fflush(stdout);

        rc = FmcjActivityInstanceNotificationVectorDeallocate(&aiNotList);
        FmcjExecutionServiceLogoff(service);
        /* Deinit API environment */
        rc= FmcjExecutionServiceDeallocate( &service );
        FmcjGlobalDisconnect();
        return -1;
      }
      printf("\n#   Creation Time     Notification for\n");
      printf("--------------------------------------\n");
      fflush(stdout);

      vectSize = FmcjActivityInstanceNotificationVectorSize(aiNotList);

      if (vectSize > 0)
      {
        for( i=0; i < vectSize; i++ )
        {
          if (i < 10) printf(" %u  ",i);
          else        printf("%u  ",i);

          aiNot = FmcjActivityInstanceNotificationVectorNextElement(aiNotList);
          timeValue = FmcjActivityInstanceNotificationCreationTime(aiNot);

          printf("%s  %s\n", FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE),
                FmcjActivityInstanceNotificationName(aiNot, tInfo, INFOSIZE));

          rc = FmcjActivityInstanceNotificationDeallocate(&aiNot);
        }
      }
      else printf("   <no activity instance notifications for user '%s' found>\n",
                FmcjServiceUserID(service, tInfo, INFOSIZE));

      /* Show actions which user can perform */
      printf("\nCommands: [a] View activity instance             [n] Reload a list\n");
      printf("              notification info                  [q] Quit program\n");
      printf("[a/n/q]? ");
      fflush(stdout);

    } /* end if: display activity instance notifications */

    /* Query user for action */
    fgets(choice,3,stdin);

    usrchoice=0;
    if (strncmp(choice,"v",1)==0) usrchoice=1;
    if (strncmp(choice,"s",1)==0) usrchoice=2;
    if (strncmp(choice,"r",1)==0) usrchoice=3;
    if (strncmp(choice,"f",1)==0) usrchoice=4;
    if (strncmp(choice,"t",1)==0) usrchoice=5;
    if (strncmp(choice,"i",1)==0) usrchoice=6;
    if (strncmp(choice,"c",1)==0) usrchoice=7;
    if (strncmp(choice,"p",1)==0) usrchoice=8;
    if (strncmp(choice,"a",1)==0) usrchoice=9;
    if (strncmp(choice,"n",1)==0) usrchoice=10;
    if (strncmp(choice,"o",1)==0) usrchoice=11;
    if (strncmp(choice,"d",1)==0) usrchoice=12;
    if (strncmp(choice,"q",1)==0) usrchoice=255;

    switch (usrchoice)
    {
      case 1:
/*-------------------------------------------------------------------*/
/* Show some workitem info                                           */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }
          FmcjItemRefresh(workitem);    /* Fetch secondary properties */

          info = FmcjWorkitemSupportTools(workitem);

          printf("Workitem name         : %s\n",
               FmcjWorkitemName(workitem, tInfo, INFOSIZE));
          printf("Process instance name : %s\n",
               FmcjWorkitemProcessInstanceName(workitem, tInfo, INFOSIZE));

          timeValue = FmcjWorkitemCreationTime(workitem);

          printf("Creation time         : %s\n",
               FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));
          printf("Program implementation: %s\n",
               FmcjWorkitemImplementation(workitem, tInfo, INFOSIZE));

          printf("Support tools         : ");

          infoVectSize = FmcjStringVectorSize(info);
          for (j=0; j < infoVectSize; j++)
          {
            FmcjStringVectorNextElement(info, aInfo, INFOSIZE);
            if   (j != (infoVectSize-1)) printf("%s, ", aInfo);
            else printf("%s\n", aInfo);
          }
          timeValue = FmcjWorkitemFirstNotificationTime(workitem);
          printf("\nFirst notification    : %s\n",
               FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));

          timeValue = FmcjWorkitemSecondNotificationTime(workitem);
          printf("Second notification   : %s\n",
               FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));
          fflush(stdout);

          /* Clear string vector */
          FmcjStringVectorDeallocate(&info);

          info = FmcjWorkitemStaff(workitem);
          infoVectSize = FmcjStringVectorSize(info);

          printf("Workitem Staff        : ");

          for (j=0; j < infoVectSize; j++)
          {
            FmcjStringVectorNextElement(info, aInfo, INFOSIZE);
            if   (j != (infoVectSize-1)) printf("%s, ", aInfo);
            else printf("%s\n", aInfo);
          }
          FmcjStringVectorDeallocate(&info);

          if (!FmcjWorkitemErrorReasonIsNull(workitem))
          {
            error = FmcjWorkitemErrorReason(workitem);
            printf("\nError RC  : %u\n", FmcjErrorRc(error));
            printf("Error text: %s\n",
                   FmcjErrorMessageText(error, tInfo, INFOSIZE));
            FmcjErrorDeallocate(&error);
          }
        }
        fflush(stdout);
        rc = FmcjWorkitemDeallocate(&workitem);

        break;


      case 2:
/*-------------------------------------------------------------------*/
/* Start a workitem                                                  */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          rc = FmcjWorkitemStart(workitem);

          if (rc != FMC_OK)
          {
            printf("\nStarting workitem failed with RC %u-%s\n\n", rc,
                   FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
        }
        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Restart workitem                                                  */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          rc = FmcjWorkitemRestart(workitem);

          if (rc != FMC_OK)
          {
            printf("\nRestarting workitem failed with RC %u-%s\n\n", rc,
                   FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
        }
        break;


      case 4:
/*-------------------------------------------------------------------*/
/* Force restart workitem                                            */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          printf("\n");
          printf("Do you want to provide an input container? [Y/N] ");
          fflush(stdout);
          scanf("%2[^\n]",tInfo);

          if (strncmp(tInfo,"Y",1)==0)
          /* ForceRestart with input container.                     */
          /* Obtain input container.                                */
          /* Insert code here to modify ("repair") container values */
          {
            rc = FmcjWorkitemInContainer(workitem, &hdlInput);
            rc = FmcjWorkitemForceRestartWithContainer(workitem, hdlInput);
          }
          else
          {
            rc = FmcjWorkitemForceRestart(workitem);
          }

          if (rc != FMC_OK)
          {
            printf("\nForced restarting workitem failed with RC %u-%s\n\n",
                   rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
          fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        }
        break;


      case 5:
/*-------------------------------------------------------------------*/
/* Terminate workitem                                                */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          rc = FmcjWorkitemTerminate(workitem);

          if (rc != FMC_OK)
          {
            printf("\nTerminating workitem failed with RC %u-%s\n\n", rc,
                   FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
        }
        break;

      case 6:
/*-------------------------------------------------------------------*/
/* Finish workitem                                                   */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          rc = FmcjWorkitemFinish(workitem);

          if (rc != FMC_OK)
          {
            printf("\nFinishing workitem failed with RC %u-%s\n\n", rc,
                   FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
        }
        break;


      case 7:
/*-------------------------------------------------------------------*/
/* Force finish workitem                                             */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          printf("\n");
          printf("Do you want to provide an output container? [Y/N] ");
          fflush(stdout);
          scanf("%2[^\n]",tInfo);

          if (strncmp(tInfo,"Y",1)==0)
          /* ForceFinish with output container.                     */
          /* Obtain output container.                               */
          /* Insert code here to modify ("repair") container values */
          {
            rc = FmcjWorkitemOutContainer( workitem, &hdlOutput );
            rc = FmcjWorkitemForceFinishWithContainer(workitem, hdlOutput);
          }
          else
          {
            rc = FmcjWorkitemForceFinish(workitem);
          }

          if (rc != FMC_OK)
          {
            printf("\nForced finishing workitem failed with RC %u-%s\n\n", rc,
                   FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
          fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        }
        break;


      case 8:
/*-------------------------------------------------------------------*/
/* Show process instance notification info                           */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"pin",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100 && vectSize > 0)
            printf("Please enter notification number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter notification number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected process instance notification */
          piNot = FmcjProcessInstanceNotificationVectorFirstElement(piNotList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjProcessInstanceNotificationDeallocate(&piNot);
            piNot = FmcjProcessInstanceNotificationVectorNextElement(piNotList);
          }

              printf("Notification for: %s\n",
                 FmcjProcessInstanceNotificationName(piNot, tInfo, INFOSIZE));

              printf("Process instance: %s\n",
                 FmcjProcessInstanceNotificationProcessInstanceName(piNot, tInfo, INFOSIZE));

          timeValue = FmcjProcessInstanceNotificationCreationTime(piNot);

          printf("Creation time   : %s\n",
               FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));

          fflush(stdout);
          rc = FmcjProcessInstanceNotificationDeallocate(&piNot);
            }
        break;


      case 9:
/*-------------------------------------------------------------------*/
/* Show activity instance notification info                          */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"ain",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1 && vectSize > 0) < 100)
            printf("Please enter notification number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter notification number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected activity instance notification */
          aiNot = FmcjActivityInstanceNotificationVectorFirstElement(aiNotList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjActivityInstanceNotificationDeallocate(&aiNot);
            aiNot = FmcjActivityInstanceNotificationVectorNextElement(aiNotList);
          }

              printf("Notification for: %s\n",
                 FmcjActivityInstanceNotificationName(aiNot, tInfo, INFOSIZE));

              printf("Activity instance: %s\n",
                 FmcjActivityInstanceNotificationProcessInstanceName(aiNot, tInfo, INFOSIZE));

          timeValue = FmcjActivityInstanceNotificationCreationTime(aiNot);

          printf("Creation time   : %s\n",
               FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));
          fflush(stdout);

          rc = FmcjActivityInstanceNotificationDeallocate(&aiNot);
        }
        break;


      case 11:
/*-------------------------------------------------------------------*/
/* Start a support tool corresponding to a workitem                  */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }
        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          if (!FmcjWorkitemSupportToolsIsNull(workitem))
          {
            info = FmcjWorkitemSupportTools(workitem);
                infoVectSize = FmcjStringVectorSize(info);

            printf("\n #  Support tool\n");
            printf("----------------\n");

            for (j=0; j < infoVectSize; j++)
            {
              FmcjStringVectorNextElement(info, aInfo, INFOSIZE);
              if   (j < 10) printf(" %u  %s", j, aInfo);
              else          printf("%u  %s", j, aInfo);
            }

            do
            {
              if ((infoVectSize-1) < 100)
                printf("Please enter support tool number (0-%u%s",infoVectSize-1,") ");
              else
                printf("Please enter support tool number (1-99)");
              fflush(stdout);

              scanf("%2[^\n]",tInfo);
              j=0;
              for (i=0; i < strlen(tInfo); i++)
              { if (isdigit(tInfo[i])) j++; }
              if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
            } while (i!=j);
            i = atoi(tInfo);
            fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
            printf("\n");
            fflush(stdout);
            if (i > (infoVectSize-1))
            {
              printf("Please use range 0-%u%s\n",infoVectSize-1,".");
            }
            else    /* user's choice is in range */
            {
              /* Get the selected support tool */
              FmcjStringVectorFirstElement(info, aInfo, INFOSIZE);
              for (j = 0; j < i; j++)
              {
                FmcjStringVectorFirstElement(info, aInfo, INFOSIZE);
              }

              printf("Starting support tool '%s'.\n",aInfo);
                  rc = FmcjWorkitemStartTool(workitem, aInfo);

                  if (rc != FMC_OK)
                  {
                printf("\nStarting support tool failed with RC %u-%s\n\n", rc,
                       FmcjResultMessageText(result, tInfo, INFOSIZE));
                fflush(stdout);
                  }
                }
            FmcjStringVectorDeallocate(&info);
          }
          else printf("\nNo support tools available for this workitem.\n");

          rc = FmcjWorkitemDeallocate(&workitem);
        }
        break;


      case 12:
/*-------------------------------------------------------------------*/
/* Delete a workitem                                                 */
/*-------------------------------------------------------------------*/
        if (strncmp(queriedObjects,"wi",1))
        {
          printf("Invalid choice!\n");
          break;
        }

        do
        {
          if ((vectSize-1) < 100)
            printf("Please enter workitem number (0-%u%s",vectSize-1,") ");
          else
            printf("Please enter workitem number (1-99)");
          fflush(stdout);

          scanf("%2[^\n]",tInfo);
          j=0;
          for (i=0; i < strlen(tInfo); i++)
          { if (isdigit(tInfo[i])) j++; }
          if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        } while (i!=j);
        i = atoi(tInfo);
        fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
        printf("\n");
        fflush(stdout);
        if (i > (vectSize-1))
        {
          printf("Please use range 0-%u%s\n",vectSize-1,".");
        }
        else    /* user's choice is in range */
        {
          /* Get the selected workitem */
          workitem = FmcjWorkitemVectorFirstElement(vwList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjWorkitemDeallocate(&workitem);
            workitem = FmcjWorkitemVectorNextElement(vwList);
          }

          rc = FmcjWorkitemDelete(workitem);

          if (rc != FMC_OK)
          {
            printf("\nDeleting workitem failed with RC %u-%s\n\n", rc,
                   FmcjResultMessageText(result, tInfo, INFOSIZE));
            fflush(stdout);
          }
          rc = FmcjWorkitemDeallocate(&workitem);
        }
        break;


    }  /* end switch */



    if (strncmp(queriedObjects,"wi",1)==0)
    {
      rc = FmcjWorkitemVectorDeallocate(&vwList);
    }
    if (strncmp(queriedObjects,"pin",1)==0)
    {
      rc = FmcjProcessInstanceNotificationVectorDeallocate(&piNotList);
    }
    if (strncmp(queriedObjects,"ain",1)==0)
    {
      rc = FmcjActivityInstanceNotificationVectorDeallocate(&aiNotList);
    }


  } while ( (usrchoice!=255) );


/*-------------------------------------------------------------------*/
/* Shutdown PEA, logoff and deinit the API environment.              */
/*-------------------------------------------------------------------*/

  if (PEAstarted)
  {
    printf("\nShutting down PEA.\n");
    rc = FmcjExecutionServicePEAShutDown(service);
  }
  printf("\nLogging off.\n");
  rc = FmcjExecutionServiceLogoff(service);

  rc= FmcjExecutionServiceDeallocate( &service );

  printf("Deinitializing API environment.\n");
  rc = FmcjGlobalDisconnect();
  return 0;
}
