// BEGIN CMVC
// ************************************************************************
//
//   Workfile: IncrementEJBUpes.java, v3-samples, fmv3
//   Last update: 01/03/23 13:42:48
//   SCCS path, id: /home/flowmark/vc/0/3/7/8/s.71 1.1
//
// ************************************************************************
// END CMVC
// BEGIN COPYRIGHT
// ************************************************************************
//
//   Licensed Materials - Property of IBM
//   WA03 SupportPac
//   (c) Copyright IBM Corp. 2001
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract
//   with IBM Corp.
//
// ************************************************************************
// END COPYRIGHT
// BEGIN DISCLAIMER
// ************************************************************************
//   DISCLAIMER
//   ----------
//   This material contains programming source code for your consideration.
//   These examples have not been thoroughly tested under all conditions.
//   IBM, therefore, cannot guarantee or imply reliability, serviceability,
//   or function of these programs.
//   ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//   WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//   NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//   FOR A PARTICULAR PURPOSE.
// ************************************************************************
// END DISCLAIMER


import java.io.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import javax.jms.*;
import javax.naming.*;
import org.apache.xerces.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import com.transarc.jmon.examples.Inc.*;


///////////////////////////////////////////////////////////////////////////////
/**
 * <p>
 * MQSeries Workflow EJB invocation sample
 * </p><p>
 * This sample shows you how to invoke an EJB using an UPES via JMS
 * </p><p>
 * This way to invoke EJBs supports transactions, but does not
 * allow you to retrieve the Workflow containers metamodel
 * </p><p>
 * This class implements the ContentHandler interface of SAX, which allows it
 * to parse XML files. This implies a lot of unused dummy methods.
 * </p><p>
 * This is not a sample on how to parse XML, it only picks out the interesting 
 * elements from the incoming XML message and ignores the rest. For a more 
 * sophisticated sample on XML parsing in a UPES, please view the &quot;Distributed 
 * process sample using XML&quot; available on the MQSeries Workflow product CD.
 * It will be installed in &lt;MQWF_ROOT&gt;\SMP\DPXML. 
 * Check there readme.html for details.
 * </p>
 */
///////////////////////////////////////////////////////////////////////////////
public class IncrementEJBUpes implements ContentHandler
{
  /** Subclass to hold the parsed data */
  private class Data
  {
    /** Name of the workflow activity */
    public String activityName    = "";
    /** Correlation ID needed to correlate the XML request with the XML response */
    public String actImplCorrelID = "";
    /** Command to be executed by this XML message */
    public String command         = "";
    /** Value of the EJB increment counter */
    public String counterValue    = "";
    /** Name of the workflow process instance */
    public String instanceName    = "";
  }

  /** Object to hold the parsed data */
  private Data                   data;
  /** XML parser */
  private XMLReader              saxParser;
  /** Remembers the latest parsed XML tag */
  private String                 latestTag;
  /** Remembers the content of the currently parsed XML element */
  private StringBuffer           currentElement;
  /** Position indicator for XML parsing */
  private Locator                locator;
  /** JMS queue connection */
  private QueueConnection        qConn;
  /** JMS queue session */
  private QueueSession           qSess;
  /** JMS queue receiver */
  private QueueReceiver          qReader;
  /** JMS queue sender */
  private QueueSender            qWriter;
  /** OS specific end of line */
  private static final String    eol = System.getProperty("line.separator");
  /** For performance measuring */
  private long                   startMillis;
  /** For performance measuring */
  private long                   stopMillis;
  /** EJB home */
  private IncHome                incrementHome  = null;





  /**
   * Receive notification of character data.
   * <p>This method is part of the SAX API.</p>
   *
   * <p>The Parser will call this method to report each chunk of
   * character data.  SAX parsers may return all contiguous character
   * data in a single chunk, or they may split it into several
   * chunks; however, all of the characters in any single event
   * must come from the same external entity so that the Locator
   * provides useful information.</p>
   *
   * <p>The application must not attempt to read from the array
   * outside of the specified range.</p>
   *
   * <p>Note that some parsers will report whitespace in element
   * content using the {@link #ignorableWhitespace ignorableWhitespace}
   * method rather than this one (validating parsers <em>must</em> 
   * do so).</p>
   *
   * @param ch The characters from the XML document.
   * @param start The start position in the array.
   * @param length The number of characters to read from the array.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see #ignorableWhitespace 
   * @see org.xml.sax.Locator
   */
  public void characters(char[] ch, int start, int length) throws SAXException
  { 
    // Just append the text to currentElement, will be evaluated in endElement(...)
    currentElement.append(new String(ch, start, length)); 
  }





  /**
   * Receive notification of the end of a document.
   * <p>This method is part of the SAX API and is here just a dummy implementation.</p>
   *
   * <p>The SAX parser will invoke this method only once, and it will
   * be the last method invoked during the parse.  The parser shall
   * not invoke this method until it has either abandoned parsing
   * (because of an unrecoverable error) or reached the end of
   * input.</p>
   *
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see #startDocument
   */
  public void endDocument() throws SAXException
  { 
  }
  




  /**
   * Receive notification of the end of an element. 
   * <p>This method is part of the SAX API.</p>
   *
   * <p>The SAX parser will invoke this method at the end of every
   * element in the XML document; there will be a corresponding
   * {@link #startElement startElement} event for every endElement 
   * event (even when the element is empty).</p>
   *
   * <p>For information on the names, see startElement.</p>
   *
   * @param uri The Namespace URI, or the empty string if the
   *        element has no Namespace URI or if Namespace
   *        processing is not being performed.
   * @param localName The local name (without prefix), or the
   *        empty string if Namespace processing is not being
   *        performed.
   * @param qName The qualified XML 1.0 name (with prefix), or the
   *        empty string if qualified names are not available.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   */
  public void endElement(String namespaceURI, String localName, String name) throws SAXException
  {
    // Trim the text received by characters(...)
    String currentString = currentElement.toString().trim();

    // Evaluate the text
    if (currentString.length() > 0)
    {
      try
      {
        // Only a few elements are of interest
        if (latestTag.equals("_ACTIVITY"))            data.activityName     = currentString;
        else if (latestTag.equals("ActImplCorrelID")) data.actImplCorrelID  = currentString;
        else if (latestTag.equals("_PROCESS"))        data.instanceName     = currentString;
        else if (latestTag.equals("Command"))         data.command          = currentString;
      }
      catch(Exception xcpt)
      {
        System.out.println("Couldn't read element '" + latestTag + "': " + currentString + "/" + currentElement);

        xcpt.printStackTrace();
      }

      // Reset for the next tag
      currentElement.setLength(0);
    }
  }





  /**
   * End the scope of a prefix-URI mapping.
   * <p>This method is part of the SAX API and is here just a dummy implementation.</p>
   *
   * <p>See {@link #startPrefixMapping startPrefixMapping} for 
   * details.  This event will always occur after the corresponding 
   * {@link #endElement endElement} event, but the order of 
   * {@link #endPrefixMapping endPrefixMapping} events is not otherwise
   * guaranteed.</p>
   *
   * @param prefix The prefix that was being mapping.
   * @exception org.xml.sax.SAXException The client may throw
   *            an exception during processing.
   * @see #startPrefixMapping
   * @see #endElement
   */
  public void endPrefixMapping(String prefix) throws SAXException
  { 
  }
  




  /**
   * Dispatcher method that actually invokes the EJB methods
   * 
   * @return The value of the increment counter in the EJB
   * 
   * @exception Exception Any occurred exception will be rethrown unfiltered
   *
   */
  private int executeCommand() throws Exception
  {
    IncKey  incrementKey; // EJB primary key
    Inc     increment;    // EJB remote interface


    // Create the primary key of the EJB, the actual key value is the process instance name
    startMillis = System.currentTimeMillis();
    incrementKey = new IncKey(data.instanceName);
    stopMillis = System.currentTimeMillis();
    System.out.println("Time consumed for creating the primary Key (ms): " + (stopMillis - startMillis));
    System.out.println("Command: " + data.command);


    // Handle the EJB create request
    startMillis = System.currentTimeMillis();
    if(data.command.equalsIgnoreCase("create"))
    {
      increment = incrementHome.create(incrementKey);
    }
    
    // Handle the EJB increment request
    if(data.command.equalsIgnoreCase("incrementandgetvalue"))
    {
      increment = incrementHome.findByPrimaryKey(incrementKey);  // Retrieve the EJB remote interface
      increment.increment();                                     // Actual EJB invocation
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for executing the command " + data.command + " (ms): " + (stopMillis - startMillis));
      return increment.getValue();                               // Actual EJB invocation
    }
    
    // Handle the EJB remove request
    if(data.command.equalsIgnoreCase("remove"))
    {
      increment = incrementHome.findByPrimaryKey(incrementKey);  // Retrieve the EJB remote interface 
      increment.remove();                                        // Actual EJB invocation
    }
    stopMillis = System.currentTimeMillis();
    System.out.println("Time consumed for executing the command " + data.command + " (ms): " + (stopMillis - startMillis));
    
    return 0;
  } // private int executeCommand() throws Exception





  /**
   * Receive notification of ignorable whitespace in element content.
   * <p>This method is part of the SAX API and is here just a dummy implementation.</p>
   *
   * <p>Validating Parsers must use this method to report each chunk
   * of whitespace in element content (see the W3C XML 1.0 recommendation,
   * section 2.10): non-validating parsers may also use this method
   * if they are capable of parsing and using content models.</p>
   *
   * <p>SAX parsers may return all contiguous whitespace in a single
   * chunk, or they may split it into several chunks; however, all of
   * the characters in any single event must come from the same
   * external entity, so that the Locator provides useful
   * information.</p>
   *
   * <p>The application must not attempt to read from the array
   * outside of the specified range.</p>
   *
   * @param ch The characters from the XML document.
   * @param start The start position in the array.
   * @param length The number of characters to read from the array.
   * @see #characters
   */
  public void ignorableWhitespace(char[] ch, int start , int length)
  {
  }





  /**
   * Constructor
   *
   * Reads the properties from file, gets the initial JNDI context,
   * initializes the JMS interface, sets up the SAX parser and gets the EJB home
   */
  public IncrementEJBUpes()
  {
    System.out.println("MQSeries Workflow EJB Invocation Example via JmsUpes");
    System.out.println("---------------------------------------------------------------");
    System.out.println("Initializing IncrementEJBUpes");
    System.out.println("Classpath: " + System.getProperty("java.class.path"));

    Properties prop = new Properties();  // Temporary needed properties

    try
    {
      // Read properties from configuration file
      startMillis = System.currentTimeMillis();
      prop.load(new FileInputStream("EJBInvocation.properties"));
      // Translate property names
      prop.put(Context.INITIAL_CONTEXT_FACTORY, prop.getProperty("EJB.INITIAL_CONTEXT_FACTORY"));
      prop.put(Context.PROVIDER_URL,            prop.getProperty("EJB.PROVIDER_URL"));
      prop.put(Context.REFERRAL, "throw");
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for retrieving the properties (ms): " + (stopMillis - startMillis));

      // Create the JNDI naming context
      startMillis = System.currentTimeMillis();
      InitialContext cxt = new InitialContext(prop);
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for getting the initial context (ms): " + (stopMillis - startMillis));

      // Initialize the JMS interface
      startMillis = System.currentTimeMillis();
      // Get the QueueConnectionFactory from JNDI
      QueueConnectionFactory factory = (QueueConnectionFactory)cxt.lookup(prop.getProperty("EJB.QUEUE_FACTORY"));
      // Initialize the JMS connection and session
      qConn   = factory.createQueueConnection();
      qSess   = qConn.createQueueSession(true, Session.AUTO_ACKNOWLEDGE);
      // Get the Queue from JNDI and create the queue receiver and sender
      qReader = qSess.createReceiver((Queue)cxt.lookup(prop.getProperty("EJB.QUEUE")));
      qWriter = qSess.createSender(null);
      // Start listening
      qConn.start();
      System.out.println("  Connecting to UPES queue '" + qReader.getQueue().getQueueName() + "' via JMS");
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for initializing the queue (ms): " + (stopMillis - startMillis));

      // Initialize the XML parser
      startMillis = System.currentTimeMillis();
      saxParser = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
      saxParser.setContentHandler(this);
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for initializing the XML parser (ms): " + (stopMillis - startMillis));


      // Retrieve the EJB home.
      startMillis = System.currentTimeMillis();
      Object obj = cxt.lookup("IncBean"); // Temporary needed dummy object
      System.out.println(obj.toString()); // See what comes, just for debugging purposes
      incrementHome = (IncHome) javax.rmi.PortableRemoteObject.narrow(obj, IncHome.class);
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for retrieving the EJB Home (ms): " + (stopMillis - startMillis));


      System.out.println("IncrementEJBUpes initialized");
    }
    catch(Exception xcpt)
    { 
      xcpt.printStackTrace(); 
      System.exit(1); 
    }
  } // public IncrementEJBUpes()
  




  /**
   * Main method, no parameter evaluation
   */
  public static void main(String[] args)
  {
    IncrementEJBUpes myself = new IncrementEJBUpes();
    myself.run();
    myself.shutdown();
    System.exit(0);
  }





  /**
   * Receive notification of a processing instruction.
   * <p>This method is part of the SAX API and is here just a dummy implementation.</p>
   *
   * <p>The Parser will invoke this method once for each processing
   * instruction found: note that processing instructions may occur
   * before or after the main document element.</p>
   *
   * <p>A SAX parser must never report an XML declaration (XML 1.0,
   * section 2.8) or a text declaration (XML 1.0, section 4.3.1)
   * using this method.</p>
   *
   * @param target The processing instruction target.
   * @param data The processing instruction data, or null if
   *        none was supplied.  The data does not include any
   *        whitespace separating it from the target.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   */
  public void processingInstruction(String target, String data) throws SAXException
  { 
  }
  




  /**
   * <p>Listener method of the UPES</p>
   * <p>Listens in a loop on the queue until a message is received.</p>
   * <p>Then the message is parsed and evaluated. If a shutdown command
   * is received, the loop is left after an empty response is sent back.</p>
   * <p>The other commands are processed and the result is sent back.</p>
   */
  private void run()
  {
    do // Until a shutdown request is received
    {
      try
      {
        System.out.println("----------------------------------------------------------------------");
        System.out.println("Waiting for next message");

        // Listen on the queue
        Message msg = qReader.receive();

        // Parse the message
        String xml = ((TextMessage)msg).getText();
        System.out.println("Message received:" + eol + xml + eol + eol);
        data = new Data();
        currentElement = new StringBuffer(1024);
        saxParser.parse(new InputSource(new StringReader(xml)));

        // Shutdown requested
        if (data.activityName.equals("ShutdownUPES"))
        {
          data.counterValue = "0"; // Dummy needed for a wellformed XML response
          writeResponse(msg);      // Write an empty response to the Workflow server
          qSess.commit();          // Commit the transaction
          break;                   // Terminate listening
        }
        // Normal command
        else
        {
          // Execute the received command
          data.counterValue = Integer.toString(executeCommand());
          // Write the response to the Workflow server
          writeResponse(msg);
        }

        // Commit the transaction
        qSess.commit();
      }
      catch(Exception xcpt) 
      { 
        xcpt.printStackTrace();
      }
    }
    while (true);

    System.out.println("QUIT received");
  } // private void run()





  /**
   * Receive an object for locating the origin of SAX document events.
   * <p>This method is part of the SAX API.</p>
   *
   * <p>SAX parsers are strongly encouraged (though not absolutely
   * required) to supply a locator: if it does so, it must supply
   * the locator to the application by invoking this method before
   * invoking any of the other methods in the ContentHandler
   * interface.</p>
   *
   * <p>The locator allows the application to determine the end
   * position of any document-related event, even if the parser is
   * not reporting an error.  Typically, the application will
   * use this information for reporting its own errors (such as
   * character content that does not match an application's
   * business rules).  The information returned by the locator
   * is probably not sufficient for use with a search engine.</p>
   *
   * <p>Note that the locator will return correct information only
   * during the invocation of the events in this interface.  The
   * application should not attempt to use it at any other time.</p>
   *
   * @param locator An object that can return the location of
   *                any SAX document event.
   * @see org.xml.sax.Locator
   */
  public void setDocumentLocator(Locator loc)
  {
    locator = loc;
  }





  /**
   * <p>Shutdown method for the UPES.</p>
   * <p>Terminates queue listening and cleans up the JMS resources.</p>
   */
  public void shutdown()
  {
    System.out.println("Shutting down IncrementEJBUpes");

    try
    {
      // Cleanup JMS resources
      if (qReader  != null) qReader.close();
      if (qWriter  != null) qWriter.close();
      if (qSess    != null) qSess.close();
      if (qConn    != null) qConn.close();
    }
    catch(Exception xcpt) 
    { 
      xcpt.printStackTrace(); 
    }

    System.out.println("IncrementEJBUpes Shutdown complete");
  }





  /**
   * Receive notification of a skipped entity.
   * <p>This method is part of the SAX API and is here just a dummy implementation.</p>
   *
   * <p>The Parser will invoke this method once for each entity
   * skipped.  Non-validating processors may skip entities if they
   * have not seen the declarations (because, for example, the
   * entity was declared in an external DTD subset).  All processors
   * may skip external entities, depending on the values of the
   * <code>http://xml.org/sax/features/external-general-entities</code>
   * and the
   * <code>http://xml.org/sax/features/external-parameter-entities</code>
   * properties.</p>
   *
   * @param name The name of the skipped entity.  If it is a 
   *        parameter entity, the name will begin with '%', and if
   *        it is the external DTD subset, it will be the string
   *        "[dtd]".
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   */
  public void skippedEntity(String name) throws SAXException
  { 
  }
  




  /**
   * Receive notification of the beginning of a document.
   * <p>This method is part of the SAX API and is here just a dummy implementation.</p>
   *
   * <p>The SAX parser will invoke this method only once, before any
   * other methods in this interface or in org.xml.sax.DTDHandler
   * DTDHandler (except for {@link #setDocumentLocator 
   * setDocumentLocator}).</p>
   *
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see #endDocument
   */
  public void startDocument() throws SAXException
  { 
  }
  




  /**
   * Receive notification of the beginning of an element.
   * <p>This method is part of the SAX API.</p>
   *
   * <p>The Parser will invoke this method at the beginning of every
   * element in the XML document; there will be a corresponding
   * {@link #endElement endElement} event for every startElement event
   * (even when the element is empty). All of the element's content will be
   * reported, in order, before the corresponding endElement
   * event.</p>
   *
   * <p>This event allows up to three name components for each
   * element:</p>
   *
   * <ol>
   * <li>the Namespace URI;</li>
   * <li>the local name; and</li>
   * <li>the qualified (prefixed) name.</li>
   * </ol>
   *
   * <p>Any or all of these may be provided, depending on the
   * values of the <var>http://xml.org/sax/features/namespaces</var>
   * and the <var>http://xml.org/sax/features/namespace-prefixes</var>
   * properties:</p>
   *
   * <ul>
   * <li>the Namespace URI and local name are required when 
   * the namespaces property is <var>true</var> (the default), and are
   * optional when the namespaces property is <var>false</var> (if one is
   * specified, both must be);</li>
   * <li>the qualified name is required when the namespace-prefixes property
   * is <var>true</var>, and is optional when the namespace-prefixes property
   * is <var>false</var> (the default).</li>
   * </ul>
   *
   * <p>Note that the attribute list provided will contain only
   * attributes with explicit values (specified or defaulted):
   * #IMPLIED attributes will be omitted.  The attribute list
   * will contain attributes used for Namespace declarations
   * (xmlns* attributes) only if the
   * <code>http://xml.org/sax/features/namespace-prefixes</code>
   * property is true (it is false by default, and support for a 
   * true value is optional).</p>
   *
   * @param uri The Namespace URI, or the empty string if the
   *        element has no Namespace URI or if Namespace
   *        processing is not being performed.
   * @param localName The local name (without prefix), or the
   *        empty string if Namespace processing is not being
   *        performed.
   * @param qName The qualified name (with prefix), or the
   *        empty string if qualified names are not available.
   * @param atts The attributes attached to the element.  If
   *        there are no attributes, it shall be an empty
   *        Attributes object.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see #endElement
   * @see org.xml.sax.Attributes
   */
  public void startElement(String namespaceURI, String localName, String name, Attributes attrs) throws SAXException
  { 
    latestTag = name; 
  }





  /**
   * Begin the scope of a prefix-URI Namespace mapping.
   * <p>This method is part of the SAX API and is here just a dummy implementation.</p>
   *
   * <p>The information from this event is not necessary for
   * normal Namespace processing: the SAX XML reader will 
   * automatically replace prefixes for element and attribute
   * names when the <code>http://xml.org/sax/features/namespaces</code>
   * feature is <var>true</var> (the default).</p>
   *
   * <p>There are cases, however, when applications need to
   * use prefixes in character data or in attribute values,
   * where they cannot safely be expanded automatically; the
   * start/endPrefixMapping event supplies the information
   * to the application to expand prefixes in those contexts
   * itself, if necessary.</p>
   *
   * <p>Note that start/endPrefixMapping events are not
   * guaranteed to be properly nested relative to each-other:
   * all startPrefixMapping events will occur before the
   * corresponding {@link #startElement startElement} event, 
   * and all {@link #endPrefixMapping endPrefixMapping}
   * events will occur after the corresponding {@link #endElement
   * endElement} event, but their order is not otherwise 
   * guaranteed.</p>
   *
   * <p>There should never be start/endPrefixMapping events for the
   * "xml" prefix, since it is predeclared and immutable.</p>
   *
   * @param prefix The Namespace prefix being declared.
   * @param uri The Namespace URI the prefix is mapped to.
   * @exception org.xml.sax.SAXException The client may throw
   *            an exception during processing.
   * @see #endPrefixMapping
   * @see #startElement
   */
  public void startPrefixMapping(String prefix, String uri) throws SAXException
  { 
  }
  




  /**
   * Formats the XML response message, sets some message properties 
   * and sends the message back to the queue specified in the input 
   * paramaters properties.
   *
   * @param msg The XML message object to which the response corresponds.
   */
  public void writeResponse(Message msg)
  {
    // Send the response message to MQSeries Workflow
    try
    {
      StringBuffer xml = new StringBuffer(1024);

      // The message is mostly static.
      xml.append("<?xml version=\"1.0\" standalone=\"yes\"?>" + eol);
      xml.append("<!-- This document generated by IncrementEJBUpes class. -->" + eol);
      xml.append("<WfMessage>" + eol);
      xml.append("<ActivityImplInvokeResponse>" + eol);
      xml.append("<ActImplCorrelID>" + data.actImplCorrelID + "</ActImplCorrelID>" + eol);
      xml.append("<ProgramRC>0</ProgramRC>" + eol);
      // Now add the container data
      xml.append("<ProgramOutputData>" + eol);
      xml.append("<EJBIncrement>" + eol);
      xml.append("  <CounterValue>" + data.counterValue + "</CounterValue>" + eol);
      xml.append("</EJBIncrement>" + eol);
      xml.append("</ProgramOutputData>" + eol);
      xml.append("</ActivityImplInvokeResponse>" + eol);
      xml.append("</WfMessage>");

      // Create the JMS response message object
      TextMessage reply = qSess.createTextMessage(xml.toString());

      // Set message properties
      reply.setJMSCorrelationID(msg.getJMSCorrelationID());

      // Retrieve the queue to which the response has to be sent
      Queue qReply = (Queue)msg.getJMSReplyTo();
      System.out.println("Connecting to REPLY queue '" + qReply.getQueueName() + "' via JMS");
      System.out.println("Replying:" + eol + xml.toString() + eol + eol);

      // Send the message
      qWriter.send(qReply, reply);
    }
    catch(Exception xcpt) 
    { 
      xcpt.printStackTrace();
    }
  } // public void writeResponse(Message msg)
}
