// *****************************************************************************
// * IBM MQSeries Workflow Java API Beans - Example Application                *
// *                                                                           *
// * Licensed Materials - Property of IBM                                      *
// * 5697-216 5697-FM3                                                         *
// * (C) Copyright IBM Corp. 1993, 1999                                        *
// * US Government Users Restricted Rights - Use, duplication or               *
// * disclosure restricted by GSA ADP Schedule Contract                        *
// * with IBM Corp.                                                            *
// *                                                                           *
// *****************************************************************************
// *                                                                           *
// * DISCLAIMER                                                                *
// *                                                                           *
// * This material contains programming source code for your consideration.    *
// * These examples have not been thoroughly tested under all conditions.      *
// * IBM, therefore, cannot guarantee or imply reliability, serviceability,    *
// * or function of these programs.                                            *
// * ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY    *
// * WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT      *
// * NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS       *
// * FOR A PARTICULAR PURPOSE.                                                 *
// *                                                                           *
// *****************************************************************************
// *                                                                           *
// * This servlet is the MQSeries Workflow Java equivalent of the typical      *
// * 'Hello World' example. The servlet first echoes the provided parameters,  *
// * then uses them to log on to the ExecutionService, retrieves the workitems *
// * of the first worklist it finds and, if any, renders them as HTML table.   *
// * After logging off the constructed response is then sent to the invoking   *
// * client.                                                                   *
// *                                                                           *
// *****************************************************************************
// * Installation: - put the HelloServlet.class file to the appropriate        *
// *                 directory of the webserver.                               *
// *               - register the servlet with the webserver using the         *
// *                 appropriate webserver administration function.            *
// *               - adapt the HelloServlet.html file to match with your       *
// *                 installation, specially the URL.                          *
// *               - put the HelloServlet.html file to the appropriate         *
// *                 directory of the webserver.                               *
// *                                                                           *
// * Usage:        - use a webbrowser and point it to the URL of your          *
// *                 Webserver. e.g. http://www.company.com/HelloServlet.html  *
// *****************************************************************************

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.workflow.api.*;

public class HelloServlet extends HttpServlet
{
  public void doPost(HttpServletRequest req, HttpServletResponse res)
  {
    // Limit requests to prevent denial of service attacks
    if (req.getContentLength() <= 8*1024)
    {
      doGet(req, res);
    }
    else
    {
      sendResponse(res, "Too big",
                   "<h1>Error - content length &gt;8k not allowed" + "</h1>");
    }
  }

  public void doGet(HttpServletRequest req, HttpServletResponse res)
  {
    try
    {
      // For servlets, usage of local bindings is highly recommended. In this
      // case, the agent name is not actually used although it must be set.
      Agent agent = new Agent();
      agent.setLocator(Agent.LOC_LOCATOR);
      agent.setName("LOCAL");

      ExecutionService service = agent.locate(req.getParameter("sysgrp"),
                                              req.getParameter("system"));

      service.logon(req.getParameter("user"), req.getParameter("password"));

      StringBuffer msg = new StringBuffer(1024);

      msg.append("<h5>SystemGroup    : " + service.systemGroupName() + "</h5>")
         .append("<h5>System         : " + service.systemName()      + "</h5>")
         .append("<h5>As User        : " + service.userID()          + "</h5>");

      WorkList[] workLists = service.queryWorkLists();

      if (workLists.length > 0)
      {
        WorkItem[] workItems = workLists[0].queryWorkItems();

        if (workItems.length > 0)
        {
          msg.append("<table border=\"1\" width=\"90%\" bgcolor=\"#C0C0C0\" bordercolor=\"#000000\">")
             .append("<tr><th width=\"50%\">WorkItem Name</th> <th width=\"50%\">Description</th></tr>");

          for (int i = 0; i < workItems.length; i++)
          {
            msg.append("<tr><td>" + workItems[i].name() + "</td><td>" + workItems[i].description() + "</td></tr>");
          }

          msg.append("</table>");
        }
        else
        {
          msg.append("<h5>No workitems in worklist</h5>");
        }
      }
      else
      {
        msg.append("<h5>No worklist found</h5>");
      }

      service.logoff();

      sendResponse(res, "Logon successful", msg.toString());
    }
    catch(java.beans.PropertyVetoException xcpt)
    {
      sendResponse(res, "Agent Error",
                   "<h1>Internal error - cannot happen in local mode</h1>");
    }
    catch(FmcException xcpt)
    {
      sendResponse(res, "API Error",
                   "<h1>API Exception:</h1>" + xcpt.messageText);
    }
  }

  private void sendResponse(HttpServletResponse response, String title, String body)
  {
    try
    {
      ServletOutputStream output = response.getOutputStream();

      response.setContentType("text/html");

      output.println("<html><head><title>MQSeries Workflow Server Response: " + title + "</title></head><body>" +
                     body + "</body></html>");
    }
    catch(IOException xcpt)
    {
    }
  }

  public String getServletInfo()
  {
    return "A servlet that shows the work items in the first worklist, if any";
  }
}
