/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Tutorial - Monitoring processes              */
/*                                                                    */
/* This sample program shows how to monitor a process with its        */
/* subordinated programs, blocks and subprocesses                     */
/**********************************************************************/

// Import necessary MQWF Java API Interface

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ActivityInstancePackage.*;
import com.ibm.workflow.api.ControlConnectorInstancePackage.*;
import com.ibm.workflow.api.ProcessInstancePackage.*;
import com.ibm.workflow.api.ServicePackage.*;

// Various needed classes

import java.io.*;
import java.util.*;

public class Monitor {

public static void main(String args[])
{

  int i = 0, j = 0;
  int aiIndex = 0;
  int cciIndex = 0;

  ProcessInstance   processInstance;
  ProcessInstance[] viList;

  ActivityInstance   activityInstance;
  ActivityInstance[] aList;

  ControlConnectorInstance   ccInstance;
  ControlConnectorInstance[] ccList;

  InstanceMonitor piMonlevel0;
  InstanceMonitor piMonlevel1;
  InstanceMonitor biMon;

  Point   point;
  Point[] pointList;

  String PIListFilter ="";
  String PIListSort ="NAME ASC";

  String choice ="";
  int    usrchoice = 0;

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
  boolean again = false;

/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (args.length!=4)
  {
    System.out.println("\nUsage: java Monitor <name of agent> <loc|jndi> <UserID> <Password>");
    System.exit(-1);
  }

  try { // general try clause for whole example

  Agent agent = new Agent();
  if      (args[1].equalsIgnoreCase("loc"))  agent.setLocator(Agent.LOC_LOCATOR);
  else if (args[1].equalsIgnoreCase("jndi")) agent.setLocator(Agent.JNDI_LOCATOR);
  else { System.out.println("Invalid locator policy: " + args[1]);
         System.exit(-1); }
  agent.setName(args[0]);
  ExecutionService service = agent.locate("","");
  System.out.println();
  service.logon2( args[2], args[3], SessionMode.DEFAULT, AbsenceIndicator.LEAVE );
  System.out.println("Logged on with UserID '" + service.userID() +
                     "' on system '" + service.systemName() +
                     "' at system group '" + service.systemGroupName() + "'.");


/*-------------------------------------------------------------------*/
/* Query available process instances                                 */
/*-------------------------------------------------------------------*/

  do                                    // Beginning of main loop
  {
    viList = service.queryProcessInstances( "", PIListSort, null );

    System.out.println("#   State        Instance name");
    System.out.println("------------------------------");

    if (viList.length>0)
    {
      for( i=0; i<viList.length; i++ )
      {
        System.out.print(i);
        if (i<10) System.out.print(" ");
        System.out.print("  ");
        switch (viList[i].state().value())
        {
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._UNDEFINED: System.out.print("undefined  "); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._READY: System.out.print("ready      "); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._RUNNING: System.out.print("running    "); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._FINISHED: System.out.print("finished   "); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._TERMINATED: System.out.print("terminated "); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._SUSPENDED: System.out.print("suspended  "); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._TERMINATING: System.out.print("terminating"); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._SUSPENDING: System.out.print("suspending "); break;
          case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._DELETED: System.out.print("deleted    "); break;
        }
        System.out.println("  " + viList[i].name());
      }
    }
    else System.out.println("    <none>");

    do
    {
      again=false;
           System.out.print("\nPlease enter the instance number you want to view the monitor of: ");
      choice = in.readLine();
      try { i = Integer.parseInt(choice); }
      catch (NumberFormatException e) { again=true; }
         } while (again);
    System.out.println();

    if (i>(viList.length-1) && viList.length>0)
    {
          System.out.println("Please use range 0-" + Integer.toString(viList.length-1) + ".\n");
    }
    else
    {
      // Obtain monitor of selected process instance

      processInstance = viList[i];

      piMonlevel0 = processInstance.obtainProcessMonitor( false );

      // Show all activity instances and control connector instances

      aList = piMonlevel0.activityInstances();

      ccList = piMonlevel0.controlConnectorInstances();

      System.out.print("\n#   Type       Activity instance name\n"
           + "-------------------------------------\n");

      // Show activity instances
      if (aList.length > 0 )
      {
        for( aiIndex=0; aiIndex<aList.length; aiIndex++ )
        {
          activityInstance = aList[aiIndex];

          System.out.print(aiIndex);
          if (aiIndex<10) System.out.print(" ");
          System.out.print("  ");

          switch (activityInstance.kind().value())
          {
           case com.ibm.workflow.api.ActivityInstancePackage.Type._NOT_SET: System.out.print("undefined"); break;
           case com.ibm.workflow.api.ActivityInstancePackage.Type._PROCESS: System.out.print("process  "); break;
           case com.ibm.workflow.api.ActivityInstancePackage.Type._PROGRAM: System.out.print("program  "); break;
           case com.ibm.workflow.api.ActivityInstancePackage.Type._BLOCK: System.out.print("block    "); break;
          }
          System.out.println("  " + activityInstance.name());
        }
      }
      else System.out.println("    <none>");

      // Show control connector instances

      System.out.print("\n#   State          Control connector instance name\n"
           + "--------------------------------------------------\n");

      if (ccList.length > 0 )
      {
        for( cciIndex=aiIndex; cciIndex<ccList.length+aiIndex; cciIndex++ )
        {
          ccInstance = ccList[cciIndex-aiIndex];

          System.out.print(cciIndex);
          if (cciIndex<10) System.out.print(" ");
          System.out.print("  ");

          switch (ccInstance.state().value())
          {
           case com.ibm.workflow.api.ControlConnectorInstancePackage.EvaluationState._IS_FALSE: System.out.print("false        "); break;
           case com.ibm.workflow.api.ControlConnectorInstancePackage.EvaluationState._IS_TRUE: System.out.print("true         "); break;
           case com.ibm.workflow.api.ControlConnectorInstancePackage.EvaluationState._NOT_EVALUATED: System.out.print("not evaluated"); break;
           case com.ibm.workflow.api.ControlConnectorInstancePackage.EvaluationState._NOT_SET: System.out.print("not set      "); break;
          }
          if (ccInstance.name()!=null)
               System.out.println("  " + ccInstance.name());
          else System.out.println("  <not set>");
        }
      }
      else System.out.println("    <none>");

      cciIndex--;

     do
     {
       again=false;
            System.out.print("\nWhich activity/control connector instance do you want to examine closer? ");
       choice = in.readLine();
       try { i = Integer.parseInt(choice); }
       catch (NumberFormatException e) { again=true; }
         } while (again);
      System.out.println();

      if ( i>cciIndex || i<0 )
      {
        System.out.println("Please use range 0-" + cciIndex + ".");
      }
      else
      {
        if (i<aiIndex)
        {
          // Examine activity instance closer

             activityInstance = aList[i];

             switch (activityInstance.kind().value())
          {
            case com.ibm.workflow.api.ActivityInstancePackage.Type._NOT_SET:
                     System.out.println("\nBecause of an unknown reason, this activity instance has not been initialised.");
              break;

            case com.ibm.workflow.api.ActivityInstancePackage.Type._PROGRAM:

                   // Showing some program information

                   System.out.print("\nActivity instance - program information\n"
                                    + "---------------------------------------\n");
              System.out.print("State          : ");
              switch (activityInstance.state().value())
              {
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._UNDEFINED: System.out.print("undefined    "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._READY: System.out.print("ready        "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._FINISHED: System.out.print("finished     "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._SUSPENDED: System.out.print("suspended    "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._CHECKED_OUT: System.out.print("checkedOut   "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._EXECUTED: System.out.print("executed     "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._FORCE_FINISHED: System.out.print("forceFinished"); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._TERMINATING: System.out.print("terminating  "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._RUNNING: System.out.print("running      "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._TERMINATED: System.out.print("terminated   "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._SKIPPED: System.out.print("skipped      "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._IN_ERROR: System.out.print("inError      "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._PLANNING: System.out.print("planning     "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._DELETED: System.out.print("deleted      "); break;
                case com.ibm.workflow.api.ActivityInstancePackage.ExecutionState._SUSPENDING: System.out.print("suspending   "); break;
               }
               System.out.println("\nImplementation : "
                    + activityInstance.implementation()
                    + "\nActivation time: "
                    + activityInstance.activationTime().getTime().toString());
              break;

            case com.ibm.workflow.api.ActivityInstancePackage.Type._PROCESS:

              // Fetch the corresponding process instance monitor

              System.out.print("\nSubprocess instance monitor information\n"
                   + "---------------------------------------\n");

              try
              {

                piMonlevel1 = piMonlevel0.obtainProcessMonitor ( activityInstance, false );

                aList = piMonlevel1.activityInstances();
                ccList = piMonlevel1.controlConnectorInstances();

                System.out.println("Number of subordinate activity instances         : "
                     + aList.length + "\n"
                     + "Number of subordinate control connector instances: "
                     + ccList.length + "\n");
              }
              catch (FmcException e)
              { System.out.println("Obtaining subprocess instances monitor failed with RC " + e.rc + " - " + e.messageText); }

              break;

            case com.ibm.workflow.api.ActivityInstancePackage.Type._BLOCK:

              // Fetch the corresponding block instance monitor

              System.out.print("\nBlock instance monitor information\n"
                   + "----------------------------------\n");

              biMon = piMonlevel0.obtainBlockMonitor( activityInstance );

              aList = biMon.activityInstances();
              ccList = biMon.controlConnectorInstances();

              System.out.println("Number of subordinate activity instances         : "
                   + aList.length
                   + "\nNumber of subordinate control connector instances: "
                   + ccList.length + "\n");
              break;
          }

        }


        else
        {
          // Examine control connector instance closer

          ccInstance = ccList[i-aiIndex];

          System.out.println("\nControl connector instance information\n"
               + "--------------------------------------\n"
               + "Source activity Oid : "
               + ccInstance.persistentOidOfSourceActivity() + "\n"
               + "Target activity Oid : "
               + ccInstance.persistentOidOfTargetActivity() + "\n"
               + "Transition Condition: "
               + ccInstance.transitionCondition() + "\n"
               + "Bend points:");

          pointList = ccInstance.bendPoints();

          if (pointList.length>0)
          {
            for (j=0; j<pointList.length; j++)
            {
              point = pointList[j];
              System.out.println("   [" + j + "] " + "X: " + point.xPosition
                   + " Y: " + point.yPosition);
            }
          }
          else
          {
            System.out.println("   <no bend points exist>");
          }

        }

      }

    } /* end else */

    System.out.print("\nDo you want to restart or quit [r/q]? ");

    choice = in.readLine();

    if (choice.equalsIgnoreCase("q")) usrchoice=255;

  } while (usrchoice!=255);


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nLogging off.");
  service.logoff();
  System.exit(0);

  } // end of general try clause

  catch(FmcException e)
  {
    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( i = 0; i < e.parameters.length ; i++)
    {
      System.out.println("    " + e.parameters[i] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();
  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
  }

} // end of void main

} // end of Monitor
