/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Handling process instances        */
/*                                                                    */
/* This example shows what can be done with process instances queried */
/* from the execution service.                                        */
/**********************************************************************/

// Import necessary MQWF Java API Interface

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ServicePackage.*;
import com.ibm.workflow.api.ProcessInstancePackage.*;

// Various needed classes

import java.io.*;
import java.util.*;

public class ProcessInstances {

public static void main (String[] args)
{

  ProcessInstance[] iList;

  int year = 0;
  int month     = 0;
  int day = 0;
  int hour = 0;
  int minute = 0;

  int threshold = 20;
  int i = 0;
  int j = 0;

  String PIListSort ="NAME ASC";

  String choice ="";
  int   usrchoice = 0;

  boolean again = false;

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (args.length!=4)
  {
    System.out.println("\nUsage: java ProcessInstances <name of agent> <loc|jndi> <UserID> <Password>");
    System.exit(-1);
  }

  try { // general try clause for whole example

  Agent agent = new Agent();
  if      (args[1].equalsIgnoreCase("loc"))  agent.setLocator(Agent.LOC_LOCATOR);
  else if (args[1].equalsIgnoreCase("jndi")) agent.setLocator(Agent.JNDI_LOCATOR);
  else { System.out.println("Invalid locator policy: " + args[1]);
         System.exit(-1); }
  agent.setName(args[0]);
  ExecutionService service = agent.locate("","");
  System.out.println();
  service.logon2( args[2], args[3], SessionMode.DEFAULT, AbsenceIndicator.LEAVE );
  System.out.println("Logged on with UserID '" + service.userID() +
                     "' on system '" + service.systemName() +
                     "' at system group '" + service.systemGroupName() + "'.");


/*--------------------------------------------------------------------*/
/* Display current instances list, query user for action              */
/*--------------------------------------------------------------------*/

  do                                    // Beginning of main tool loop
  {
    // Display as many process instances as specified with threshold

    System.out.println("\nAvailable process instances:\n\n");

    iList = service.queryProcessInstances( "", PIListSort, new Integer(threshold));

    System.out.print("#   State        Instance name\n"
                       + "------------------------------\n");

    if (iList.length > 0 )
    {
      for( i=0; i< iList.length; i++ )
      {
        System.out.print(i);
        if (i<10) System.out.print(" ");
        System.out.print("  ");
        switch (iList[i].state().value())
        {
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._UNDEFINED: System.out.print("undefined  "); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._READY: System.out.print("ready      "); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._RUNNING: System.out.print("running    "); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._FINISHED: System.out.print("finished   "); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._TERMINATED: System.out.print("terminated "); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._SUSPENDED: System.out.print("suspended  "); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._TERMINATING: System.out.print("terminating"); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._SUSPENDING: System.out.print("suspending "); break;
         case com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._DELETED: System.out.print("deleted    "); break;
        }
        System.out.println("  " + iList[i].name());
      }
    }
    else System.out.println("  <no instances available>");

    System.out.print("\nCommands: [v] View instance information             [t] Terminate instance\n"
         + "          [s] Suspend instance                      [n] Start instance\n"
         + "          [r] Resume instance\n"
         + "          [a] Restart instance                      [q] Quit program\n"
         + "[v/s/r/a/t/n/q]? ");

    choice = in.readLine();

    usrchoice=0;

    if (choice.equalsIgnoreCase("v")) usrchoice=1;
    if (choice.equalsIgnoreCase("s")) usrchoice=2;
    if (choice.equalsIgnoreCase("r")) usrchoice=3;
    if (choice.equalsIgnoreCase("a")) usrchoice=4;
    if (choice.equalsIgnoreCase("t")) usrchoice=5;
    if (choice.equalsIgnoreCase("n")) usrchoice=6;
    if (choice.equalsIgnoreCase("q")) usrchoice=255;

    if (iList.length>0)
    switch (usrchoice)
    {

      case 1:
/*-------------------------------------------------------------------*/
/* Show some instance properties                                     */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter instance number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(iList.length-1) && iList.length>0)
        {
          System.out.println("Please use range 0-" + Integer.toString(iList.length-1) + ".\n");
        }
        else
        {
          System.out.println("Instance properties\n"
                             + "-------------------\n"
                             + "   Name     : " + iList[i].name()
                             + "\n   Object id: " + iList[i].persistentOid());
          if (iList[i].suspensionExpirationTime()!=null)
          System.out.println("   Suspension expiration time: "
                             + iList[i].suspensionExpirationTime().getTime().toString());
          else System.out.println("   Suspension expiration time: <not set>");
   }

        break;


      case 2:
/*-------------------------------------------------------------------*/
/* Demonstrates how to suspend a process instance.                   */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter instance number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(iList.length-1) && iList.length>0)
        {
     System.out.println("Please use range 0-" + Integer.toString(iList.length-1) + ".\n");
        }
        else                            // user's choice is in range
        {
     Calendar stime = Calendar.getInstance();
     stime.setTime(new Date());
     System.out.println("Current time: " + stime.getTime().toString() + "\n");
     System.out.print("Do you want to set a suspension expiration time? [y/n] ");

          choice = in.readLine();

          if (choice.equalsIgnoreCase("y"))
          {
            System.out.print("Enter year  : "); choice = in.readLine();
       try { year = Integer.parseInt(choice); } catch (NumberFormatException e) { year=stime.YEAR; }
            System.out.print("Enter month : "); choice = in.readLine();
       try { month = Integer.parseInt(choice); } catch (NumberFormatException e) { month=stime.MONTH; }
            System.out.print("Enter day   : "); choice = in.readLine();
       try { day = Integer.parseInt(choice); } catch (NumberFormatException e) { day=stime.DAY_OF_MONTH; }
            System.out.print("Enter hour  : "); choice = in.readLine();
       try { hour = Integer.parseInt(choice); } catch (NumberFormatException e) { hour=stime.HOUR; }
            System.out.print("Enter minute: "); choice = in.readLine();
       try { minute = Integer.parseInt(choice); } catch (NumberFormatException e) { minute=stime.MINUTE; }

            stime.set(year, month, day, hour, minute, 0);

            // Suspend the process instance and all subordinated processes
            // and let them be resumed automatically at the specified time.

            try { iList[i].suspend2( stime, true ); }
            catch (FmcException e)
            { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }

          }
          else
          {
            // Suspend the process instance and all subordinated processes

       try { iList[i].suspend( true ); }
            catch (FmcException e)
            { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
          }

   } // end else: user's choice is in range

   break;


      case 3:
/*-------------------------------------------------------------------*/
/* Demonstrates how to resume a process instance.                    */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter instance number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(iList.length-1) && iList.length>0)
        {
     System.out.println("Please use range 0-" + Integer.toString(iList.length-1) + ".\n");
        }
        else                            // user's choice is in range
        {
          // Resume the process instance and all subordinated processes
          try { iList[i].resume( true ); }
     catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
   }

   break;


      case 4:
/*-------------------------------------------------------------------*/
/* Demonstrates how to restart a process instance.                   */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter instance number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(iList.length-1) && iList.length>0)
        {
     System.out.println("Please use range 0-" + Integer.toString(iList.length-1) + ".\n");
        }
        else                            // user's choice is in range
        {
          try { iList[i].restart(); }
     catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
   }
   break;


      case 5:
/*-------------------------------------------------------------------*/
/* Demonstrates how to terminate a process instance.                 */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter instance number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(iList.length-1) && iList.length>0)
        {
     System.out.println("Please use range 0-" + Integer.toString(iList.length-1) + ".\n");
        }
        else                            // user's choice is in range
        {
          try { iList[i].terminate(); }
     catch (FmcException e)
     { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
   }
   break;


      case 6:
/*-------------------------------------------------------------------*/
/* Demonstrates how to start a process instance.                     */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter instance number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(iList.length-1) && iList.length>0)
        {
     System.out.println("Please use range 0-" + Integer.toString(iList.length-1) + ".\n");
        }
        else                            // user's choice is in range
        {
          // Start instance without passing an input container
          try { iList[i].start(); }
     catch (FmcException e)
     { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
   }
   break;

  }                             // end of switch
  else System.out.println("\nNo instances available!\n");

  } while ( (usrchoice!=255) );         // end of main tool loop


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nLogging off.");
  service.logoff();
  System.exit(0);

  } // end of general try clause

  catch(FmcException e)
  {
    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( i = 0; i < e.parameters.length ; i++)
    {
      System.out.println("    " + e.parameters[i] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();
  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
  }

} // end of void main

} // end of ProcessInstances
